#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_fabric_span_session_source_filter
version_added: "2.11.0"
short_description: Manage Fabric SPAN Sessions Source Filter on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Switched Port Analyzer (SPAN) Sessions Source Filter on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.4) and later.
- This module allows for creation and deletion only; updates are not supported.
author:
- Sabari Jaganathan (@sajagana)
options:
  template:
    description:
    - The name of the template.
    - The template must be a Fabric Monitoring Access Policy template.
    - This parameter or O(template_id) is required.
    type: str
  template_id:
    description:
    - The ID of the template.
    - The template must be a Fabric Monitoring Access Policy template.
    - This parameter or O(template) is required.
    type: str
  span_session_name:
    description:
    - The name of the SPAN Session.
    - This parameter or O(span_session_uuid) is required.
    type: str
  span_session_uuid:
    description:
    - The UUID of the SPAN Session.
    - This parameter or O(span_session_name) is required.
    type: str
  span_session_source_name:
    description:
    - The name of the SPAN Session source.
    type: str
  source_ip_prefix:
    description:
    - The source IP prefix for the SPAN Session source filter.
    - This filters traffic based on the source IP address.
    - This can be a valid IPv4 or IPv6 address.
    type: str
  destination_ip_prefix:
    description:
    - The destination IP prefix for the SPAN Session source filter.
    - This filters traffic based on the destination IP address.
    - This can be a valid IPv4 or IPv6 address.
    type: str
  source_port_from:
    description:
    - The starting source port number for the SPAN Session source filter.
    - This parameter is required to query/delete a specific SPAN session source filter when not configured with "unspecified".
    type: str
    default: unspecified
    choices: ['53', 'dns', '20', 'ftp_data', '80', 'http', '443', 'https', '110', 'pop3', '554', 'rtsp', '25', 'smtp', '22', 'ssh', '0', 'unspecified']
  source_port_to:
    description:
    - The ending source port number for the SPAN Session source filter.
    - This parameter is required to query/delete a specific SPAN session source filter when not configured with "unspecified".
    type: str
    default: unspecified
    choices: ['53', 'dns', '20', 'ftp_data', '80', 'http', '443', 'https', '110', 'pop3', '554', 'rtsp', '25', 'smtp', '22', 'ssh', '0', 'unspecified']
  destination_port_from:
    description:
    - The starting destination port number for the SPAN Session source filter.
    - This parameter is required to query/delete a specific SPAN session source filter when not configured with "unspecified".
    type: str
    default: unspecified
    choices: ['53', 'dns', '20', 'ftp_data', '80', 'http', '443', 'https', '110', 'pop3', '554', 'rtsp', '25', 'smtp', '22', 'ssh', '0', 'unspecified']
  destination_port_to:
    description:
    - The ending destination port number for the SPAN Session source filter.
    - This parameter is required to query/delete a specific SPAN session source filter when not configured with "unspecified".
    type: str
    default: unspecified
    choices: ['53', 'dns', '20', 'ftp_data', '80', 'http', '443', 'https', '110', 'pop3', '554', 'rtsp', '25', 'smtp', '22', 'ssh', '0', 'unspecified']
  ip_protocol:
    description:
    - The IP protocol for the SPAN Session source filter.
    - This filters traffic based on the Layer 3 or Layer 4 protocol.
    - This parameter is required to query/delete a specific SPAN session source filter when not configured with "unspecified".
    type: str
    choices:
      - '0'
      - 'unspecified'
      - 'egp'
      - '8'
      - 'eigrp'
      - '88'
      - 'icmp'
      - '1'
      - 'icmpv6'
      - '58'
      - 'igmp'
      - '2'
      - 'igp'
      - '9'
      - 'l2tp'
      - '115'
      - 'ospfigp'
      - '89'
      - 'pim'
      - '103'
      - 'tcp'
      - '6'
      - 'udp'
      - '17'
    default: unspecified
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Fabric Monitoring Access Policy template.
- The O(span_session_name) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_fabric_span_session) to create the Fabric SPAN Session.
- The O(span_session_source_name) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_fabric_span_session_source) to create the Fabric SPAN Session Source.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.ndo_fabric_span_session
- module: cisco.mso.ndo_fabric_span_session_source
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create SPAN Session source with IPv4 address
  cisco.mso.ndo_fabric_span_session_source_filter:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_mon_pol
    span_session_name: ansible_test_span_session
    span_session_source_name: ansible_test_source_1
    source_ip_prefix: 1.1.1.1
    destination_ip_prefix: 2.2.2.2
    source_port_from: http
    source_port_to: https
    destination_port_from: http
    destination_port_to: https
    ip_protocol: tcp
    state: present

- name: Create SPAN Session source filter with IPv6 address
  cisco.mso.ndo_fabric_span_session_source_filter:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_mon_pol
    span_session_name: ansible_test_span_session
    span_session_source_name: ansible_test_source_1
    source_ip_prefix: 1::1
    destination_ip_prefix: 2::2
    source_port_from: 20
    source_port_to: 22
    destination_port_from: 25
    destination_port_to: 110
    ip_protocol: 6
    state: present

- name: Query a specific SPAN Session source filter
  cisco.mso.ndo_fabric_span_session_source_filter:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_mon_pol
    span_session_name: ansible_test_span_session
    span_session_source_name: ansible_test_source_1
    source_ip_prefix: 1.1.1.1
    destination_ip_prefix: 2.2.2.2
    source_port_from: "ftp_data"
    source_port_to: "ssh"
    destination_port_from: "smtp"
    destination_port_to: "pop3"
    ip_protocol: "tcp"
    state: query
  register: query_one

- name: Query all SPAN Session source filters
  cisco.mso.ndo_fabric_span_session_source_filter:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_mon_pol
    span_session_name: ansible_test_span_session
    span_session_source_name: ansible_test_source_1
    state: query
  register: query_all

- name: Delete a specific SPAN Session source filter
  cisco.mso.ndo_fabric_span_session_source_filter:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_mon_pol
    span_session_name: ansible_test_span_session
    span_session_source_name: ansible_test_source_1
    source_ip_prefix: 1.1.1.1
    destination_ip_prefix: 2.2.2.2
    source_port_from: 20
    source_port_to: 22
    destination_port_from: 25
    destination_port_to: 110
    ip_protocol: 6
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate
from ansible_collections.cisco.mso.plugins.module_utils.constants import PORT_MAPPING, IP_PROTOCOL_MAPPING
from collections import namedtuple
import copy

port_mapping_values = list(PORT_MAPPING) + list(PORT_MAPPING.values())
ip_protocol_mapping_values = list(IP_PROTOCOL_MAPPING) + list(IP_PROTOCOL_MAPPING.values())


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str"),
        template_id=dict(type="str"),
        span_session_name=dict(type="str"),
        span_session_uuid=dict(type="str"),
        span_session_source_name=dict(type="str"),
        source_ip_prefix=dict(type="str"),
        destination_ip_prefix=dict(type="str"),
        source_port_from=dict(type="str", choices=port_mapping_values, default="unspecified"),
        source_port_to=dict(type="str", choices=port_mapping_values, default="unspecified"),
        destination_port_from=dict(type="str", choices=port_mapping_values, default="unspecified"),
        destination_port_to=dict(type="str", choices=port_mapping_values, default="unspecified"),
        ip_protocol=dict(type="str", choices=ip_protocol_mapping_values, default="unspecified"),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ("template", "template_id"),
            ("span_session_name", "span_session_uuid"),
        ],
        required_if=[
            ["state", "absent", ["span_session_source_name", "source_ip_prefix", "destination_ip_prefix"]],
            ["state", "present", ["span_session_source_name", "source_ip_prefix", "destination_ip_prefix"]],
        ],
        required_one_of=[
            ["template", "template_id"],
            ["span_session_name", "span_session_uuid"],
        ],
    )

    mso = MSOModule(module)

    template_name = module.params.get("template")
    template_id = module.params.get("template_id")
    span_session_name = module.params.get("span_session_name")
    span_session_uuid = module.params.get("span_session_uuid")
    span_session_source_name = module.params.get("span_session_source_name")
    source_ip_prefix = module.params.get("source_ip_prefix")
    destination_ip_prefix = module.params.get("destination_ip_prefix")
    source_port_from = int(PORT_MAPPING.get(module.params.get("source_port_from"), module.params.get("source_port_from")))
    source_port_to = int(PORT_MAPPING.get(module.params.get("source_port_to"), module.params.get("source_port_to")))
    destination_port_from = int(PORT_MAPPING.get(module.params.get("destination_port_from"), module.params.get("destination_port_from")))
    destination_port_to = int(PORT_MAPPING.get(module.params.get("destination_port_to"), module.params.get("destination_port_to")))
    ip_protocol = IP_PROTOCOL_MAPPING.get(module.params.get("ip_protocol"), module.params.get("ip_protocol"))
    state = module.params.get("state")

    mso_template = MSOTemplate(mso, "monitoring_tenant", template_name, template_id)
    mso_template.validate_template("monitoring")

    fabric_span_session = mso_template.get_fabric_span_session(span_session_uuid, span_session_name, fail_module=True)
    span_session_source = mso_template.get_fabric_span_session_source(
        span_session_source_name, fabric_span_session.details.get("sourceGroup", {}).get("sources", []), fail_module=True
    )

    match = get_fabric_span_session_source_filter(mso_template, module.params, span_session_source.details.get("filters"))

    filter_object_full_config = dict(
        templateId=mso_template.template_id,
        templateName=mso_template.template_name,
        spanSessionName=fabric_span_session.details.get("name"),
        spanSessionUUID=fabric_span_session.details.get("uuid"),
        spanSessionSourceName=span_session_source.details.get("name"),
    )

    if match and source_ip_prefix and destination_ip_prefix:
        match.details.update(filter_object_full_config)
        mso.existing = mso.previous = copy.copy(match.details)  # Query a specific object
    elif match:
        for filter in match:
            filter.update(filter_object_full_config)
        mso.existing = match  # Query all objects

    if state != "query":
        span_session_source_filter_path = "/monitoringTemplate/template/spanSessions/{0}/sourceGroup/sources/{1}/filters/{2}".format(
            fabric_span_session.index, span_session_source.index, match.index if match else "-"
        )

    ops = []

    if state == "present" and not match:
        mso_values = dict(
            srcIPPrefix=source_ip_prefix,
            srcPortFrom=source_port_from,
            srcPortTo=source_port_to,
            destIPPrefix=destination_ip_prefix,
            destPortFrom=destination_port_from,
            destPortTo=destination_port_to,
            ipProtocol=ip_protocol,
        )

        mso.sanitize(copy.copy(mso_values))
        ops.append(dict(op="add", path=span_session_source_filter_path, value=mso_values))

    elif state == "absent" and match:
        ops.append(dict(op="remove", path=span_session_source_filter_path))

    if not module.check_mode and ops:
        if state == "present" and not match or state == "absent" and match:
            mso_template.template = mso.request(mso_template.template_path, method="PATCH", data=ops)
            fabric_span_session = mso_template.get_fabric_span_session(span_session_uuid, span_session_name, fail_module=True)
            span_session_source = mso_template.get_fabric_span_session_source(
                span_session_source_name, fabric_span_session.details.get("sourceGroup", {}).get("sources", []), fail_module=True
            )
            match = get_fabric_span_session_source_filter(mso_template, module.params, span_session_source.details.get("filters"))
            if match:
                match.details.update(filter_object_full_config)
                mso.existing = match.details  # When the state is present
            else:
                mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        if state == "present":
            mso.existing = copy.copy(mso.proposed)
            mso.existing.update(filter_object_full_config)
        else:
            mso.existing = {}

    mso.exit_json()


def get_fabric_span_session_source_filter(mso_template, filter_config, search_list, fail_module=False):
    KVPair = namedtuple("KVPair", "key value")
    if filter_config and filter_config.get("source_ip_prefix") and filter_config.get("destination_ip_prefix") and search_list:  # Query a specific object
        KVPairs = [
            KVPair("srcIPPrefix", filter_config.get("source_ip_prefix")),
            KVPair("srcPortFrom", int(PORT_MAPPING.get(filter_config.get("source_port_from"), filter_config.get("source_port_from")))),
            KVPair("srcPortTo", int(PORT_MAPPING.get(filter_config.get("source_port_to"), filter_config.get("source_port_to")))),
            KVPair("destIPPrefix", filter_config.get("destination_ip_prefix")),
            KVPair("destPortFrom", int(PORT_MAPPING.get(filter_config.get("destination_port_from"), filter_config.get("destination_port_from")))),
            KVPair("destPortTo", int(PORT_MAPPING.get(filter_config.get("destination_port_to"), filter_config.get("destination_port_to")))),
            KVPair("ipProtocol", IP_PROTOCOL_MAPPING.get(filter_config.get("ip_protocol"), filter_config.get("ip_protocol"))),
        ]
        return mso_template.get_object_by_key_value_pairs("SPAN Session Source Filter", search_list, KVPairs, fail_module)
    return search_list  # Query all objects


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_appliance_ssids
short_description: Resource module for networks _appliance _ssids
description:
  - Manage operation update of the resource networks _appliance _ssids.
  - Update the attributes of an MX SSID.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  authMode:
    description: The association control method for the SSID ('open', 'psk', '8021x-meraki'
      or '8021x-radius').
    type: str
  defaultVlanId:
    description: The VLAN ID of the VLAN associated to this SSID. This parameter is
      only valid if the network is in routed mode.
    type: int
  dhcpEnforcedDeauthentication:
    description: DHCP Enforced Deauthentication enables the disassociation of wireless
      clients in addition to Mandatory DHCP. This param is only valid on firmware
      versions >= MX 17.0 where the associated LAN has Mandatory DHCP Enabled.
    suboptions:
      enabled:
        description: Enable DCHP Enforced Deauthentication on the SSID.
        type: bool
    type: dict
  dot11w:
    description: The current setting for Protected Management Frames (802.11w).
    suboptions:
      enabled:
        description: Whether 802.11w is enabled or not.
        type: bool
      required:
        description: (Optional) Whether 802.11w is required or not.
        type: bool
    type: dict
  enabled:
    description: Whether or not the SSID is enabled.
    type: bool
  encryptionMode:
    description: The psk encryption mode for the SSID ('wep' or 'wpa'). This param
      is only valid if the authMode is 'psk'.
    type: str
  name:
    description: The name of the SSID.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  number:
    description: Number path parameter.
    type: str
  psk:
    description: The passkey for the SSID. This param is only valid if the authMode
      is 'psk'.
    type: str
  radiusServers:
    description: The RADIUS 802.1x servers to be used for authentication. This param
      is only valid if the authMode is '8021x-radius'.
    elements: dict
    suboptions:
      host:
        description: The IP address of your RADIUS server.
        type: str
      port:
        description: The UDP port your RADIUS servers listens on for Access-requests.
        type: int
      secret:
        description: The RADIUS client shared secret.
        type: str
    type: list
  visible:
    description: Boolean indicating whether the MX should advertise or hide this SSID.
    type: bool
  wpaEncryptionMode:
    description: The types of WPA encryption. ('WPA1 and WPA2', 'WPA2 only', 'WPA3
      Transition Mode' or 'WPA3 only'). This param is only valid if (1) the authMode
      is 'psk' & the encryptionMode is 'wpa' OR (2) the authMode is '8021x-meraki'
      OR (3) the authMode is '8021x-radius'.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance updateNetworkApplianceSsid
    description: Complete reference of the updateNetworkApplianceSsid API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-ssid
notes:
  - SDK Method used are
    appliance.Appliance.update_network_appliance_ssid,
  - Paths used are
    put /networks/{networkId}/appliance/ssids/{number},
"""

EXAMPLES = r"""
- name: Update by id
  cisco.meraki.networks_appliance_ssids:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    authMode: 8021x-radius
    defaultVlanId: 1
    dhcpEnforcedDeauthentication:
      enabled: true
    dot11w:
      enabled: true
      required: true
    enabled: true
    encryptionMode: wpa
    name: My SSID
    networkId: string
    number: string
    psk: psk
    radiusServers:
      - host: 0.0.0.0
        port: 1000
        secret: secret
    visible: true
    wpaEncryptionMode: WPA2 only
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "number": 0,
      "name": "string",
      "enabled": true,
      "defaultVlanId": 0,
      "authMode": "string",
      "radiusServers": [
        {
          "host": "string",
          "port": 0
        }
      ],
      "encryptionMode": "string",
      "wpaEncryptionMode": "string",
      "visible": true
    }
"""



import asyncio
import datetime
import json
from typing import TYPE_CHECKING, Any, cast

from aiobotocore.session import get_session
from botocore.client import BaseClient

if TYPE_CHECKING:
    from collections.abc import Awaitable

DOCUMENTATION = r"""
---
short_description: Receive events from an AWS CloudTrail
description:
  - An ansible-rulebook event source module for getting events from an AWS CloudTrail.
  - >
    This supports all the authentication methods supported by boto3 library:
    U(https://boto3.amazonaws.com/v1/documentation/api/latest/guide/credentials.html).
options:
  access_key:
    description:
      - Optional AWS access key ID.
    type: str
  secret_key:
    description:
      - Optional AWS secret access key.
    type: str
  session_token:
    description:
      - Optional STS session token for use with temporary credentials.
    type: str
  endpoint_url:
    description:
      - Optional URL to connect to instead of the default AWS endpoints.
    type: str
  region:
    description:
      - Optional AWS region to use.
    type: str
  delay_seconds:
    description:
      - The number of seconds to wait between polling.
    type: int
    default: 10
  lookup_attributes:
    description:
      - The optional list of lookup attributes.
      - A lookup attribute is a dictionary containing an AttributeKey (string),
        which specifies the attribute used to filter returned events, and an
        AttributeValue (string), which defines the value for the specified AttributeKey.
    type: list
    elements: str
  event_category:
    description:
      - The optional event category to return. (e.g. 'insight')
    type: str
"""

EXAMPLES = r"""
- amazon.aws.aws_cloudtrail:
    region: us-east-1
    lookup_attributes:
      - AttributeKey: 'EventSource'
        AttributeValue: 'ec2.amazonaws.com'
      - AttributeKey: 'ReadOnly'
        AttributeValue: 'true'
    event_category: management
"""


def _cloudtrail_event_to_dict(event: dict[str, Any]) -> dict[str, Any]:
    event["EventTime"] = event["EventTime"].isoformat()
    event["CloudTrailEvent"] = json.loads(event["CloudTrailEvent"])
    return event


def _get_events(
    events: list[dict[str, Any]],
    last_event_ids: list[str],
) -> tuple[list[dict[str, Any]], Any, list[str]]:
    event_time = None
    event_ids = []
    result = []
    for event in events:
        # skip last event
        if last_event_ids and event["EventId"] in last_event_ids:
            continue
        if event_time is None or event_time < event["EventTime"]:
            event_time = event["EventTime"]
            event_ids = [event["EventId"]]
        elif event_time == event["EventTime"]:
            event_ids.append(event["EventId"])
        result.append(event)
    return result, event_time, event_ids


async def _get_cloudtrail_events(
    client: BaseClient,
    params: dict[str, Any],
) -> list[dict[str, Any]]:
    paginator = client.get_paginator("lookup_events")
    results: dict[str, Any] = await cast(
        "Awaitable[dict[str, Any]]",
        paginator.paginate(**params).build_full_result(),
    )
    events = results.get("Events", [])
    # type guards:
    if not isinstance(events, list):
        err_msg = "Events is not a list"
        raise TypeError(err_msg)
    for event in events:
        if not isinstance(event, dict):
            err_msg = "Event is not a dictionary"
            raise TypeError(err_msg)
    return events


ARGS_MAPPING = {
    "lookup_attributes": "LookupAttributes",
    "event_category": "EventCategory",
}


async def main(queue: asyncio.Queue[Any], args: dict[str, Any]) -> None:
    """Receive events via AWS CloudTrail."""
    delay = int(args.get("delay_seconds", 10))

    session = get_session()
    params = {}
    for key, value in ARGS_MAPPING.items():
        if args.get(key) is not None:
            params[value] = args.get(key)

    params["StartTime"] = datetime.datetime.now(tz=datetime.timezone.utc)

    async with session.create_client("cloudtrail", **connection_args(args)) as client:
        event_time = None
        event_ids: list[str] = []
        while True:
            if event_time is not None:
                params["StartTime"] = event_time
            events = await _get_cloudtrail_events(client, params)

            events, c_event_time, c_event_ids = _get_events(events, event_ids)
            for event in events:
                await queue.put(_cloudtrail_event_to_dict(event))

            event_ids = c_event_ids or event_ids
            event_time = c_event_time or event_time

            await asyncio.sleep(delay)


def connection_args(args: dict[str, Any]) -> dict[str, Any]:
    """Provide connection arguments to AWS CloudTrail."""
    selected_args = {}

    # Best Practice: get credentials from ~/.aws/credentials or the environment
    # However the following method are still possible
    if "access_key" in args:
        selected_args["aws_access_key_id"] = args["access_key"]
    if "secret_key" in args:
        selected_args["aws_secret_access_key"] = args["secret_key"]
    if "session_token" in args:
        selected_args["aws_session_token"] = args["session_token"]

    if "endpoint_url" in args:
        selected_args["endpoint_url"] = args["endpoint_url"]
    if "region" in args:
        selected_args["region_name"] = args["region"]
    return selected_args


if __name__ == "__main__":
    """MockQueue if running directly."""

    class MockQueue(asyncio.Queue[Any]):
        """A fake queue."""

        async def put(self: "MockQueue", event: dict[str, Any]) -> None:
            """Print the event."""
            print(event)  # noqa: T201

    asyncio.run(main(MockQueue(), {}))



#!/usr/bin/python

# Copyright (C) 2018 IBM CORPORATION
# Author(s): Tzur Eliyahu <tzure@il.ibm.com>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ibm_sa_vol_map
short_description: Handles volume mapping on IBM Spectrum Accelerate Family storage systems

description:
  - This module maps volumes to or unmaps them from the hosts on IBM Spectrum Accelerate Family storage systems.
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  vol:
    description:
      - Volume name.
    required: true
    type: str
  state:
    default: "present"
    choices: ["present", "absent"]
    description:
      - When the state is present the volume is mapped. When the state is absent, the volume is meant to be unmapped.
    type: str

  cluster:
    description:
      - Maps the volume to a cluster.
    type: str
  host:
    description:
      - Maps the volume to a host.
    type: str
  lun:
    description:
      - The LUN identifier.
    type: str
  override:
    description:
      - Overrides the existing volume mapping.
    type: str

extends_documentation_fragment:
  - community.general._ibm_storage
  - community.general._attributes

author:
  - Tzur Eliyahu (@tzure)
"""

EXAMPLES = r"""
- name: Map volume to host.
  community.general.ibm_sa_vol_map:
    vol: volume_name
    lun: 1
    host: host_name
    username: admin
    password: secret
    endpoints: hostdev-system
    state: present

- name: Map volume to cluster.
  community.general.ibm_sa_vol_map:
    vol: volume_name
    lun: 1
    cluster: cluster_name
    username: admin
    password: secret
    endpoints: hostdev-system
    state: present

- name: Unmap volume.
  community.general.ibm_sa_vol_map:
    host: host_name
    username: admin
    password: secret
    endpoints: hostdev-system
    state: absent
"""
RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._ibm_sa_utils import (
    connect_ssl,
    execute_pyxcli_command,
    is_pyxcli_installed,
    spectrum_accelerate_spec,
)


def main():
    argument_spec = spectrum_accelerate_spec()
    argument_spec.update(
        dict(
            state=dict(default="present", choices=["present", "absent"]),
            vol=dict(required=True),
            lun=dict(),
            cluster=dict(),
            host=dict(),
            override=dict(),
        )
    )

    module = AnsibleModule(argument_spec)
    is_pyxcli_installed(module)

    xcli_client = connect_ssl(module)
    # required args
    mapping = False
    try:
        mapped_hosts = xcli_client.cmd.vol_mapping_list(vol=module.params.get("vol")).as_list
        for host in mapped_hosts:
            if host["host"] == module.params.get("host", ""):
                mapping = True
    except Exception:
        pass
    state = module.params["state"]

    state_changed = False
    if state == "present" and not mapping:
        state_changed = execute_pyxcli_command(module, "map_vol", xcli_client)
    if state == "absent" and mapping:
        state_changed = execute_pyxcli_command(module, "unmap_vol", xcli_client)

    module.exit_json(changed=state_changed)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2018, Evert Mulder (base on manageiq_user.py by Daniel Korn <korndaniel1@gmail.com>)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: manageiq_tenant

short_description: Management of tenants in ManageIQ
extends_documentation_fragment:
  - community.general._manageiq
  - community.general._attributes

author: Evert Mulder (@evertmulder)
description:
  - The manageiq_tenant module supports adding, updating and deleting tenants in ManageIQ.
requirements:
  - manageiq-client
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  state:
    type: str
    description:
      - V(absent) - tenant should not exist,
      - V(present) - tenant should be.
    choices: ['absent', 'present']
    default: 'present'
  name:
    type: str
    description:
      - The tenant name.
    required: true
    default:
  description:
    type: str
    description:
      - The tenant description.
    required: true
    default:
  parent_id:
    type: int
    description:
      - The ID of the parent tenant. If not supplied the root tenant is used.
      - The O(parent_id) takes president over O(parent) when supplied.
    default:
  parent:
    type: str
    description:
      - The name of the parent tenant. If not supplied and no O(parent_id) is supplied the root tenant is used.
    default:
  quotas:
    type: dict
    description:
      - The tenant quotas.
      - All parameters case sensitive.
      - 'Valid attributes are:'
      - '- V(cpu_allocated) (int): use null to remove the quota.'
      - '- V(mem_allocated) (GB): use null to remove the quota.'
      - '- V(storage_allocated) (GB): use null to remove the quota.'
      - '- V(vms_allocated) (int): use null to remove the quota.'
      - '- V(templates_allocated) (int): use null to remove the quota.'
    default: {}
"""

EXAMPLES = r"""
- name: Update the root tenant in ManageIQ
  community.general.manageiq_tenant:
    name: 'My Company'
    description: 'My company name'
    manageiq_connection:
      url: 'http://127.0.0.1:3000'
      username: 'admin'
      password: 'smartvm'
      validate_certs: false # only do this when you trust the network!

- name: Create a tenant in ManageIQ
  community.general.manageiq_tenant:
    name: 'Dep1'
    description: 'Manufacturing department'
    parent_id: 1
    manageiq_connection:
      url: 'http://127.0.0.1:3000'
      username: 'admin'
      password: 'smartvm'
      validate_certs: false # only do this when you trust the network!

- name: Delete a tenant in ManageIQ
  community.general.manageiq_tenant:
    state: 'absent'
    name: 'Dep1'
    parent_id: 1
    manageiq_connection:
      url: 'http://127.0.0.1:3000'
      username: 'admin'
      password: 'smartvm'
      validate_certs: false # only do this when you trust the network!

- name: Set tenant quota for cpu_allocated, mem_allocated, remove quota for vms_allocated
  community.general.manageiq_tenant:
    name: 'Dep1'
    parent_id: 1
    quotas:
      - cpu_allocated: 100
      - mem_allocated: 50
      - vms_allocated:
    manageiq_connection:
      url: 'http://127.0.0.1:3000'
      username: 'admin'
      password: 'smartvm'
      validate_certs: false # only do this when you trust the network!


- name: Delete a tenant in ManageIQ using a token
  community.general.manageiq_tenant:
    state: 'absent'
    name: 'Dep1'
    parent_id: 1
    manageiq_connection:
      url: 'http://127.0.0.1:3000'
      token: 'sometoken'
      validate_certs: false # only do this when you trust the network!
"""

RETURN = r"""
tenant:
  description: The tenant.
  returned: success
  type: complex
  contains:
    id:
      description: The tenant ID.
      returned: success
      type: int
    name:
      description: The tenant name.
      returned: success
      type: str
    description:
      description: The tenant description.
      returned: success
      type: str
    parent_id:
      description: The ID of the parent tenant.
      returned: success
      type: int
    quotas:
      description: List of tenant quotas.
      returned: success
      type: list
      sample:
        cpu_allocated: 100
        mem_allocated: 50
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._manageiq import ManageIQ, manageiq_argument_spec


class ManageIQTenant:
    """
    Object to execute tenant management operations in manageiq.
    """

    def __init__(self, manageiq):
        self.manageiq = manageiq

        self.module = self.manageiq.module
        self.api_url = self.manageiq.api_url
        self.client = self.manageiq.client

    def tenant(self, name, parent_id, parent):
        """Search for tenant object by name and parent_id or parent
            or the root tenant if no parent or parent_id is supplied.
        Returns:
            the parent tenant, None for the root tenant
            the tenant or None if tenant was not found.
        """

        if parent_id:
            parent_tenant_res = self.client.collections.tenants.find_by(id=parent_id)
            if not parent_tenant_res:
                self.module.fail_json(msg=f"Parent tenant with id '{parent_id}' not found in manageiq")
            parent_tenant = parent_tenant_res[0]
            tenants = self.client.collections.tenants.find_by(name=name)

            for tenant in tenants:
                try:
                    ancestry = tenant["ancestry"]
                except AttributeError:
                    ancestry = None

                if ancestry:
                    tenant_parent_id = int(ancestry.split("/")[-1])
                    if int(tenant_parent_id) == parent_id:
                        return parent_tenant, tenant

            return parent_tenant, None
        else:
            if parent:
                parent_tenant_res = self.client.collections.tenants.find_by(name=parent)
                if not parent_tenant_res:
                    self.module.fail_json(msg=f"Parent tenant '{parent}' not found in manageiq")

                if len(parent_tenant_res) > 1:
                    self.module.fail_json(msg=f"Multiple parent tenants not found in manageiq with name '{parent}'")

                parent_tenant = parent_tenant_res[0]
                parent_id = int(parent_tenant["id"])
                tenants = self.client.collections.tenants.find_by(name=name)

                for tenant in tenants:
                    try:
                        ancestry = tenant["ancestry"]
                    except AttributeError:
                        ancestry = None

                    if ancestry:
                        tenant_parent_id = int(ancestry.split("/")[-1])
                        if tenant_parent_id == parent_id:
                            return parent_tenant, tenant

                return parent_tenant, None
            else:
                # No parent or parent id supplied we select the root tenant
                return None, self.client.collections.tenants.find_by(ancestry=None)[0]

    def compare_tenant(self, tenant, name, description):
        """Compare tenant fields with new field values.

        Returns:
            false if tenant fields have some difference from new fields, true o/w.
        """
        found_difference = (name and tenant["name"] != name) or (description and tenant["description"] != description)

        return not found_difference

    def delete_tenant(self, tenant):
        """Deletes a tenant from manageiq.

        Returns:
            dict with `msg` and `changed`
        """
        try:
            url = f"{self.api_url}/tenants/{tenant['id']}"
            result = self.client.post(url, action="delete")
        except Exception as e:
            self.module.fail_json(msg=f"failed to delete tenant {tenant['name']}: {e}")

        if result["success"] is False:
            self.module.fail_json(msg=result["message"])

        return dict(changed=True, msg=result["message"])

    def edit_tenant(self, tenant, name, description):
        """Edit a manageiq tenant.

        Returns:
            dict with `msg` and `changed`
        """
        resource = dict(name=name, description=description, use_config_for_attributes=False)

        # check if we need to update ( compare_tenant is true is no difference found )
        if self.compare_tenant(tenant, name, description):
            return dict(changed=False, msg=f"tenant {tenant['name']} is not changed.", tenant=tenant["_data"])

        # try to update tenant
        try:
            self.client.post(tenant["href"], action="edit", resource=resource)
        except Exception as e:
            self.module.fail_json(msg=f"failed to update tenant {tenant['name']}: {e}")

        return dict(changed=True, msg=f"successfully updated the tenant with id {tenant['id']}")

    def create_tenant(self, name, description, parent_tenant):
        """Creates the tenant in manageiq.

        Returns:
            dict with `msg`, `changed` and `tenant_id`
        """
        parent_id = parent_tenant["id"]
        # check for required arguments
        for key, value in dict(name=name, description=description, parent_id=parent_id).items():
            if value in (None, ""):
                self.module.fail_json(msg=f"missing required argument: {key}")

        url = f"{self.api_url}/tenants"

        resource = {"name": name, "description": description, "parent": {"id": parent_id}}

        try:
            result = self.client.post(url, action="create", resource=resource)
            tenant_id = result["results"][0]["id"]
        except Exception as e:
            self.module.fail_json(msg=f"failed to create tenant {name}: {e}")

        return dict(
            changed=True, msg=f"successfully created tenant '{name}' with id '{tenant_id}'", tenant_id=tenant_id
        )

    def tenant_quota(self, tenant, quota_key):
        """Search for tenant quota object by tenant and quota_key.
        Returns:
            the quota for the tenant, or None if the tenant quota was not found.
        """

        tenant_quotas = self.client.get(f"{tenant['href']}/quotas?expand=resources&filter[]=name={quota_key}")

        return tenant_quotas["resources"]

    def tenant_quotas(self, tenant):
        """Search for tenant quotas object by tenant.
        Returns:
            the quotas for the tenant, or None if no tenant quotas were not found.
        """

        tenant_quotas = self.client.get(f"{tenant['href']}/quotas?expand=resources")

        return tenant_quotas["resources"]

    def update_tenant_quotas(self, tenant, quotas):
        """Creates the tenant quotas in manageiq.

        Returns:
            dict with `msg` and `changed`
        """

        changed = False
        messages = []
        for quota_key, quota_value in quotas.items():
            current_quota_filtered = self.tenant_quota(tenant, quota_key)
            if current_quota_filtered:
                current_quota = current_quota_filtered[0]
            else:
                current_quota = None

            if quota_value:
                # Change the byte values to GB
                if quota_key in ["storage_allocated", "mem_allocated"]:
                    quota_value_int = int(quota_value) * 1024 * 1024 * 1024
                else:
                    quota_value_int = int(quota_value)
                if current_quota:
                    res = self.edit_tenant_quota(tenant, current_quota, quota_key, quota_value_int)
                else:
                    res = self.create_tenant_quota(tenant, quota_key, quota_value_int)
            else:
                if current_quota:
                    res = self.delete_tenant_quota(tenant, current_quota)
                else:
                    res = dict(changed=False, msg=f"tenant quota '{quota_key}' does not exist")

            if res["changed"]:
                changed = True

            messages.append(res["msg"])

        return dict(changed=changed, msg=", ".join(messages))

    def edit_tenant_quota(self, tenant, current_quota, quota_key, quota_value):
        """Update the tenant quotas in manageiq.

        Returns:
            result
        """

        if current_quota["value"] == quota_value:
            return dict(changed=False, msg=f"tenant quota {quota_key} already has value {quota_value}")
        else:
            url = f"{tenant['href']}/quotas/{current_quota['id']}"
            resource = {"value": quota_value}
            try:
                self.client.post(url, action="edit", resource=resource)
            except Exception as e:
                self.module.fail_json(msg=f"failed to update tenant quota {quota_key}: {e}")

            return dict(changed=True, msg=f"successfully updated tenant quota {quota_key}")

    def create_tenant_quota(self, tenant, quota_key, quota_value):
        """Creates the tenant quotas in manageiq.

        Returns:
            result
        """
        url = f"{tenant['href']}/quotas"
        resource = {"name": quota_key, "value": quota_value}
        try:
            self.client.post(url, action="create", resource=resource)
        except Exception as e:
            self.module.fail_json(msg=f"failed to create tenant quota {quota_key}: {e}")

        return dict(changed=True, msg=f"successfully created tenant quota {quota_key}")

    def delete_tenant_quota(self, tenant, quota):
        """deletes the tenant quotas in manageiq.

        Returns:
            result
        """
        try:
            result = self.client.post(quota["href"], action="delete")
        except Exception as e:
            self.module.fail_json(msg=f"failed to delete tenant quota '{quota['name']}': {e}")

        return dict(changed=True, msg=result["message"])

    def create_tenant_response(self, tenant, parent_tenant):
        """Creates the ansible result object from a manageiq tenant entity

        Returns:
            a dict with the tenant id, name, description, parent id,
            quota's
        """
        tenant_quotas = self.create_tenant_quotas_response(tenant["tenant_quotas"])

        try:
            ancestry = tenant["ancestry"]
            tenant_parent_id = ancestry.split("/")[-1]
        except AttributeError:
            # The root tenant does not return the ancestry attribute
            tenant_parent_id = None

        return dict(
            id=tenant["id"],
            name=tenant["name"],
            description=tenant["description"],
            parent_id=tenant_parent_id,
            quotas=tenant_quotas,
        )

    @staticmethod
    def create_tenant_quotas_response(tenant_quotas):
        """Creates the ansible result object from a manageiq tenant_quotas entity

        Returns:
            a dict with the applied quotas, name and value
        """

        if not tenant_quotas:
            return {}

        result = {}
        for quota in tenant_quotas:
            if quota["unit"] == "bytes":
                value = float(quota["value"]) / (1024 * 1024 * 1024)
            else:
                value = quota["value"]
            result[quota["name"]] = value
        return result


def main():
    argument_spec = dict(
        name=dict(required=True, type="str"),
        description=dict(required=True, type="str"),
        parent_id=dict(type="int"),
        parent=dict(type="str"),
        state=dict(choices=["absent", "present"], default="present"),
        quotas=dict(type="dict", default={}),
    )
    # add the manageiq connection arguments to the arguments
    argument_spec.update(manageiq_argument_spec())

    module = AnsibleModule(argument_spec=argument_spec)

    name = module.params["name"]
    description = module.params["description"]
    parent_id = module.params["parent_id"]
    parent = module.params["parent"]
    state = module.params["state"]
    quotas = module.params["quotas"]

    manageiq = ManageIQ(module)
    manageiq_tenant = ManageIQTenant(manageiq)

    parent_tenant, tenant = manageiq_tenant.tenant(name, parent_id, parent)

    # tenant should not exist
    if state == "absent":
        # if we have a tenant, delete it
        if tenant:
            res_args = manageiq_tenant.delete_tenant(tenant)
        # if we do not have a tenant, nothing to do
        else:
            if parent_id:
                msg = f"tenant '{name}' with parent_id {int(parent_id)} does not exist in manageiq"
            else:
                msg = f"tenant '{name}' with parent '{parent}' does not exist in manageiq"

            res_args = dict(changed=False, msg=msg)

    # tenant should exist
    if state == "present":
        # if we have a tenant, edit it
        if tenant:
            res_args = manageiq_tenant.edit_tenant(tenant, name, description)

        # if we do not have a tenant, create it
        else:
            res_args = manageiq_tenant.create_tenant(name, description, parent_tenant)
            tenant = manageiq.client.get_entity("tenants", res_args["tenant_id"])

        # quotas as supplied and we have a tenant
        if quotas:
            tenant_quotas_res = manageiq_tenant.update_tenant_quotas(tenant, quotas)
            if tenant_quotas_res["changed"]:
                res_args["changed"] = True
                res_args["tenant_quotas_msg"] = tenant_quotas_res["msg"]

        tenant.reload(expand="resources", attributes=["tenant_quotas"])
        res_args["tenant"] = manageiq_tenant.create_tenant_response(tenant, parent_tenant)

    module.exit_json(**res_args)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_switch_dscp_to_cos_mappings
short_description: Resource module for networks _switch _dscp _to _cos _mappings
description:
  - Manage operation update of the resource networks _switch _dscp _to _cos _mappings.
  - Update the DSCP to CoS mappings.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  mappings:
    description: An array of DSCP to CoS mappings. An empty array will reset the mappings
      to default.
    elements: dict
    suboptions:
      cos:
        description: The actual layer-2 CoS queue the DSCP value is mapped to. These
          are not bits set on outgoing frames. Value can be in the range of 0 to 5
          inclusive.
        type: int
      dscp:
        description: The Differentiated Services Code Point (DSCP) tag in the IP header
          that will be mapped to a particular Class-of-Service (CoS) queue. Value
          can be in the range of 0 to 63 inclusive.
        type: int
      title:
        description: Label for the mapping (optional).
        type: str
    type: list
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch updateNetworkSwitchDscpToCosMappings
    description: Complete reference of the updateNetworkSwitchDscpToCosMappings API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-dscp-to-cos-mappings
notes:
  - SDK Method used are
    switch.Switch.update_network_switch_dscp_to_cos_mappings,
  - Paths used are
    put /networks/{networkId}/switch/dscpToCosMappings,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_switch_dscp_to_cos_mappings:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    mappings:
      - cos: 1
        dscp: 1
        title: Video
    networkId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    [
      {
        "dscp": 0,
        "cos": 0,
        "title": "string"
      }
    ]
"""



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The nxos lacp fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.lacp.lacp import (
    LacpArgs,
)


class LacpFacts(object):
    """The nxos lacp fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = LacpArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for lacp
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """
        if not data:
            data = connection.get("show running-config | include lacp")
        resources = data.strip()
        objs = self.render_config(self.generated_spec, resources)
        ansible_facts["ansible_network_resources"].pop("lacp", None)
        facts = {}
        if objs:
            params = utils.validate_config(self.argument_spec, {"config": objs})
            facts["lacp"] = utils.remove_empties(params["config"])
        ansible_facts["ansible_network_resources"].update(facts)
        return ansible_facts

    def render_config(self, spec, conf):
        """
        Render config as dictionary structure and delete keys
          from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)

        p_match = re.search(r"lacp system-priority (\d+)", conf, re.M)
        if p_match:
            config["system"]["priority"] = p_match.group(1)

        a_match = re.search(r"lacp system-mac (\S+)", conf, re.M)
        if a_match:
            address = a_match.group(1)
            config["system"]["mac"]["address"] = address
            r_match = re.search(r"lacp system-mac {0} role (\S+)".format(address), conf, re.M)
            if r_match:
                config["system"]["mac"]["role"] = r_match.group(1)

        return utils.remove_empties(config)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_config_snapshot
short_description: Manage Config Snapshots (config:Snapshot and config:ExportP)
description:
- Manage Config Snapshots on Cisco ACI fabrics.
- Creating new Snapshots is done using the configExportP class.
- Removing Snapshots is done using the configSnapshot class.
options:
  description:
    description:
    - The description for the Config Export Policy.
    type: str
    aliases: [ descr ]
  export_policy:
    description:
    - The name of the Export Policy to use for Config Snapshots.
    type: str
    aliases: [ name ]
  format:
    description:
    - Sets the config backup to be formatted in JSON or XML.
    - The APIC defaults to C(json) when unset.
    type: str
    choices: [ json, xml ]
  include_secure:
    description:
    - Determines if secure information should be included in the backup.
    - The APIC defaults to C(true) when unset.
    type: bool
  max_count:
    description:
    - Determines how many snapshots can exist for the Export Policy before the APIC starts to rollover.
    - Accepted values range between C(1) and C(10).
    - The APIC defaults to C(3) when unset.
    type: int
  snapshot:
    description:
    - The name of the snapshot to delete.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The APIC does not provide a mechanism for naming the snapshots.
- 'Snapshot files use the following naming structure: ce_<config export policy name>-<yyyy>-<mm>-<dd>T<hh>:<mm>:<ss>.<mss>+<hh>:<mm>.'
- 'Snapshot objects use the following naming structure: run-<yyyy>-<mm>-<dd>T<hh>-<mm>-<ss>.'
- It is strongly recommended to add a pause task after creating a Snapshot.
- Wait for the Snapshot to be finished before querying, comparing Snapshots or processing to Rollbacks.
seealso:
- module: cisco.aci.aci_config_rollback
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(config:Snapshot) and B(config:ExportP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Jacob McGill (@jmcgill298)
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Create a Snapshot
  cisco.aci.aci_config_snapshot:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: present
    export_policy: config_backup
    max_count: 10
    description: Backups taken before new configs are applied.
  delegate_to: localhost

- name: Wait for snapshot to be finished before querying
  pause:
    seconds: 10

- name: Query all Snapshots
  cisco.aci.aci_config_snapshot:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Query Snapshots associated with a particular Export Policy
  cisco.aci.aci_config_snapshot:
    host: apic
    username: admin
    password: SomeSecretPassword
    export_policy: config_backup
    state: query
  delegate_to: localhost
  register: query_result

- name: Delete a Snapshot
  cisco.aci.aci_config_snapshot:
    host: apic
    username: admin
    password: SomeSecretPassword
    export_policy: config_backup
    snapshot: run-2017-08-24T17-20-05
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

import json

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        description=dict(type="str", aliases=["descr"]),
        export_policy=dict(type="str", aliases=["name"]),  # Not required for querying all objects
        format=dict(type="str", choices=["json", "xml"]),
        include_secure=dict(type="bool"),
        max_count=dict(type="int"),
        snapshot=dict(type="str"),
        state=dict(type="str", choices=["absent", "present", "query"], default="present"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=False,
        required_if=[
            ["state", "absent", ["export_policy", "snapshot"]],
            ["state", "present", ["export_policy"]],
        ],
    )

    aci = ACIModule(module)

    description = module.params.get("description")
    export_policy = module.params.get("export_policy")
    file_format = module.params.get("format")
    include_secure = aci.boolean(module.params.get("include_secure"))
    max_count = module.params.get("max_count")
    if max_count is not None:
        if max_count in range(1, 11):
            max_count = str(max_count)
        else:
            module.fail_json(msg="Parameter 'max_count' must be a number between 1 and 10")
    snapshot = module.params.get("snapshot")
    if snapshot is not None and not snapshot.startswith("run-"):
        snapshot = "run-" + snapshot
    state = module.params.get("state")

    if state == "present":
        aci.construct_url(
            root_class=dict(
                aci_class="configExportP",
                aci_rn="fabric/configexp-{0}".format(export_policy),
                module_object=export_policy,
                target_filter={"name": export_policy},
            ),
        )

        aci.get_existing()

        aci.payload(
            aci_class="configExportP",
            class_config=dict(
                adminSt="triggered",
                descr=description,
                format=file_format,
                includeSecureFields=include_secure,
                maxSnapshotCount=max_count,
                name=export_policy,
                snapshot="yes",
            ),
        )

        aci.get_diff("configExportP")

        # Create a new Snapshot
        aci.post_config()

        # Query for job information and add to results
        path = "api/node/mo/uni/backupst/jobs-[uni/fabric/configexp-{0}].json".format(export_policy)
        resp, info = aci.api_call("GET", "{0}/{1}".format(aci.base_url, path), return_response=True)
        try:
            aci.imdata = json.loads(resp.read()).get("imdata")
        except AttributeError:
            aci.imdata = json.loads(info.get("body")).get("imdata")
        aci.result["job_details"] = {}
        if aci.imdata and isinstance(aci.imdata, list) and isinstance(aci.imdata[0], dict):
            aci.result["job_details"] = aci.imdata[0].get("configJobCont", {})

    else:
        # Prefix the proper url to export_policy
        if export_policy is not None:
            export_policy = "uni/fabric/configexp-{0}".format(export_policy)

        aci.construct_url(
            root_class=dict(
                aci_class="configSnapshotCont",
                aci_rn="backupst/snapshots-[{0}]".format(export_policy),
                module_object=export_policy,
                target_filter={"name": export_policy},
            ),
            subclass_1=dict(
                aci_class="configSnapshot",
                aci_rn="snapshot-{0}".format(snapshot),
                module_object=snapshot,
                target_filter={"name": snapshot},
            ),
        )

        aci.get_existing()

        if state == "absent":
            # Build POST request to used to remove Snapshot
            aci.payload(
                aci_class="configSnapshot",
                class_config=dict(
                    name=snapshot,
                    retire="yes",
                ),
            )

            if aci.existing:
                aci.get_diff("configSnapshot")

                # Mark Snapshot for Deletion
                aci.post_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The ios_acl_interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.acl_interfaces.acl_interfaces import (
    Acl_interfacesArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.acl_interfaces import (
    Acl_interfacesTemplate,
)


class Acl_interfacesFacts(object):
    """The ios_acl_interfaces fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Acl_interfacesArgs.argument_spec

    def get_acl_interfaces_data(self, connection):
        return connection.get(
            "show running-config | include ^interface|ip access-group|ipv6 traffic-filter",
        )

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for interfaces
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """
        if not data:
            data = self.get_acl_interfaces_data(connection)

        config_parser = Acl_interfacesTemplate(lines=data.splitlines())
        entry = sorted(list(config_parser.parse().values()), key=lambda k, sk="name": k[sk])
        if entry:
            for item in entry:
                item["access_groups"] = sorted(
                    list(item["access_groups"].values()),
                    key=lambda k, sk="afi": k[sk],
                )

        ansible_facts["ansible_network_resources"].pop("acl_interfaces", None)
        facts = {"acl_interfaces": []}
        for cfg in entry:
            utils.remove_empties(cfg)
            if cfg.get("access_groups"):
                facts["acl_interfaces"].append(cfg)
        utils.validate_config(self.argument_spec, {"config": facts.get("acl_interfaces")})

        ansible_facts["ansible_network_resources"].update(facts)
        return ansible_facts



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
from ansible.plugins.action import ActionBase
try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator, )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.meraki.plugins.plugin_utils.meraki import (
    MERAKI,
    meraki_argument_spec,
    meraki_compare_equality2,
    get_dict_result,
)
from ansible_collections.cisco.meraki.plugins.plugin_utils.exceptions import (
    InconsistentParameters,
)

# Get common arguments specification
argument_spec = meraki_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    state=dict(type="str", default="present", choices=["present"]),
    name=dict(type="str"),
    enabled=dict(type="bool"),
    authMode=dict(type="str"),
    enterpriseAdminAccess=dict(type="str"),
    ssidAdminAccessible=dict(type="bool"),
    encryptionMode=dict(type="str"),
    psk=dict(type="str"),
    wpaEncryptionMode=dict(type="str"),
    dot11w=dict(type="dict"),
    dot11r=dict(type="dict"),
    splashPage=dict(type="str"),
    splashGuestSponsorDomains=dict(type="list"),
    oauth=dict(type="dict"),
    localRadius=dict(type="dict"),
    ldap=dict(type="dict"),
    activeDirectory=dict(type="dict"),
    radiusServers=dict(type="list"),
    radiusProxyEnabled=dict(type="bool"),
    radiusTestingEnabled=dict(type="bool"),
    radiusCalledStationId=dict(type="str"),
    radiusAuthenticationNasId=dict(type="str"),
    radiusServerTimeout=dict(type="int"),
    radiusServerAttemptsLimit=dict(type="int"),
    radiusFallbackEnabled=dict(type="bool"),
    radiusRadsec=dict(type="dict"),
    radiusCoaEnabled=dict(type="bool"),
    radiusFailoverPolicy=dict(type="str"),
    radiusLoadBalancingPolicy=dict(type="str"),
    radiusAccountingEnabled=dict(type="bool"),
    radiusAccountingServers=dict(type="list"),
    radiusAccountingInterimInterval=dict(type="int"),
    radiusAttributeForGroupPolicies=dict(type="str"),
    ipAssignmentMode=dict(type="str"),
    useVlanTagging=dict(type="bool"),
    concentratorNetworkId=dict(type="str"),
    secondaryConcentratorNetworkId=dict(type="str"),
    disassociateClientsOnVpnFailover=dict(type="bool"),
    vlanId=dict(type="int"),
    defaultVlanId=dict(type="int"),
    apTagsAndVlanIds=dict(type="list"),
    walledGardenEnabled=dict(type="bool"),
    walledGardenRanges=dict(type="list"),
    gre=dict(type="dict"),
    radiusOverride=dict(type="bool"),
    radiusGuestVlanEnabled=dict(type="bool"),
    radiusGuestVlanId=dict(type="int"),
    minBitrate=dict(type="float"),
    bandSelection=dict(type="str"),
    perClientBandwidthLimitUp=dict(type="int"),
    perClientBandwidthLimitDown=dict(type="int"),
    perSsidBandwidthLimitUp=dict(type="int"),
    perSsidBandwidthLimitDown=dict(type="int"),
    lanIsolationEnabled=dict(type="bool"),
    visible=dict(type="bool"),
    availableOnAllAps=dict(type="bool"),
    availabilityTags=dict(type="list"),
    adaptivePolicyGroupId=dict(type="str"),
    mandatoryDhcpEnabled=dict(type="bool"),
    adultContentFilteringEnabled=dict(type="bool"),
    dnsRewrite=dict(type="dict"),
    speedBurst=dict(type="dict"),
    namedVlans=dict(type="dict"),
    localAuthFallback=dict(type="dict"),
    radiusAccountingStartDelay=dict(type="int"),
    networkId=dict(type="str"),
    number=dict(type="str"),
))

required_if = [
    ("state", "present", ["name", "networkId", "number"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class NetworksWirelessSsids(object):
    def __init__(self, params, meraki):
        self.meraki = meraki
        self.new_object = dict(
            name=params.get("name"),
            enabled=params.get("enabled"),
            authMode=params.get("authMode"),
            enterpriseAdminAccess=params.get("enterpriseAdminAccess"),
            ssidAdminAccessible=params.get("ssidAdminAccessible"),
            encryptionMode=params.get("encryptionMode"),
            psk=params.get("psk"),
            wpaEncryptionMode=params.get("wpaEncryptionMode"),
            dot11w=params.get("dot11w"),
            dot11r=params.get("dot11r"),
            splashPage=params.get("splashPage"),
            splashGuestSponsorDomains=params.get("splashGuestSponsorDomains"),
            oauth=params.get("oauth"),
            localRadius=params.get("localRadius"),
            ldap=params.get("ldap"),
            activeDirectory=params.get("activeDirectory"),
            radiusServers=params.get("radiusServers"),
            radiusProxyEnabled=params.get("radiusProxyEnabled"),
            radiusTestingEnabled=params.get("radiusTestingEnabled"),
            radiusCalledStationId=params.get("radiusCalledStationId"),
            radiusAuthenticationNasId=params.get("radiusAuthenticationNasId"),
            radiusServerTimeout=params.get("radiusServerTimeout"),
            radiusServerAttemptsLimit=params.get("radiusServerAttemptsLimit"),
            radiusFallbackEnabled=params.get("radiusFallbackEnabled"),
            radiusRadsec=params.get("radiusRadsec"),
            radiusCoaEnabled=params.get("radiusCoaEnabled"),
            radiusFailoverPolicy=params.get("radiusFailoverPolicy"),
            radiusLoadBalancingPolicy=params.get("radiusLoadBalancingPolicy"),
            radiusAccountingEnabled=params.get("radiusAccountingEnabled"),
            radiusAccountingServers=params.get("radiusAccountingServers"),
            radiusAccountingInterimInterval=params.get("radiusAccountingInterimInterval"),
            radiusAttributeForGroupPolicies=params.get("radiusAttributeForGroupPolicies"),
            ipAssignmentMode=params.get("ipAssignmentMode"),
            useVlanTagging=params.get("useVlanTagging"),
            concentratorNetworkId=params.get("concentratorNetworkId"),
            secondaryConcentratorNetworkId=params.get("secondaryConcentratorNetworkId"),
            disassociateClientsOnVpnFailover=params.get("disassociateClientsOnVpnFailover"),
            vlanId=params.get("vlanId"),
            defaultVlanId=params.get("defaultVlanId"),
            apTagsAndVlanIds=params.get("apTagsAndVlanIds"),
            walledGardenEnabled=params.get("walledGardenEnabled"),
            walledGardenRanges=params.get("walledGardenRanges"),
            gre=params.get("gre"),
            radiusOverride=params.get("radiusOverride"),
            radiusGuestVlanEnabled=params.get("radiusGuestVlanEnabled"),
            radiusGuestVlanId=params.get("radiusGuestVlanId"),
            minBitrate=params.get("minBitrate"),
            bandSelection=params.get("bandSelection"),
            perClientBandwidthLimitUp=params.get("perClientBandwidthLimitUp"),
            perClientBandwidthLimitDown=params.get("perClientBandwidthLimitDown"),
            perSsidBandwidthLimitUp=params.get("perSsidBandwidthLimitUp"),
            perSsidBandwidthLimitDown=params.get("perSsidBandwidthLimitDown"),
            lanIsolationEnabled=params.get("lanIsolationEnabled"),
            visible=params.get("visible"),
            availableOnAllAps=params.get("availableOnAllAps"),
            availabilityTags=params.get("availabilityTags"),
            adaptivePolicyGroupId=params.get("adaptivePolicyGroupId"),
            mandatoryDhcpEnabled=params.get("mandatoryDhcpEnabled"),
            adultContentFilteringEnabled=params.get("adultContentFilteringEnabled"),
            dnsRewrite=params.get("dnsRewrite"),
            speedBurst=params.get("speedBurst"),
            namedVlans=params.get("namedVlans"),
            localAuthFallback=params.get("localAuthFallback"),
            radiusAccountingStartDelay=params.get("radiusAccountingStartDelay"),
            networkId=params.get("networkId"),
            number=params.get("number"),
        )

    def get_all_params(self, name=None, id=None):
        new_object_params = {}
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        return new_object_params

    def get_params_by_id(self, name=None, id=None):
        new_object_params = {}
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        if self.new_object.get('number') is not None or self.new_object.get(
                'number') is not None:
            new_object_params['number'] = self.new_object.get('number')
        return new_object_params

    def update_by_id_params(self):
        new_object_params = {}
        if self.new_object.get('name') is not None or self.new_object.get(
                'name') is not None:
            new_object_params['name'] = self.new_object.get('name') or \
                self.new_object.get('name')
        if self.new_object.get('enabled') is not None or self.new_object.get(
                'enabled') is not None:
            new_object_params['enabled'] = self.new_object.get('enabled')
        if self.new_object.get('authMode') is not None or self.new_object.get(
                'auth_mode') is not None:
            new_object_params['authMode'] = self.new_object.get(
                'authMode') or self.new_object.get('auth_mode')
        if self.new_object.get('enterpriseAdminAccess') is not None or self.new_object.get(
                'enterprise_admin_access') is not None:
            new_object_params['enterpriseAdminAccess'] = self.new_object.get(
                'enterpriseAdminAccess') or self.new_object.get('enterprise_admin_access')
        if self.new_object.get('ssidAdminAccessible') is not None or self.new_object.get(
                'ssid_admin_accessible') is not None:
            new_object_params['ssidAdminAccessible'] = self.new_object.get(
                'ssidAdminAccessible')
        if self.new_object.get('encryptionMode') is not None or self.new_object.get(
                'encryption_mode') is not None:
            new_object_params['encryptionMode'] = self.new_object.get(
                'encryptionMode') or self.new_object.get('encryption_mode')
        if self.new_object.get(
                'psk') is not None or self.new_object.get('psk') is not None:
            new_object_params['psk'] = self.new_object.get('psk') or \
                self.new_object.get('psk')
        if self.new_object.get('wpaEncryptionMode') is not None or self.new_object.get(
                'wpa_encryption_mode') is not None:
            new_object_params['wpaEncryptionMode'] = self.new_object.get(
                'wpaEncryptionMode') or self.new_object.get('wpa_encryption_mode')
        if self.new_object.get('dot11w') is not None or self.new_object.get(
                'dot11w') is not None:
            new_object_params['dot11w'] = self.new_object.get('dot11w') or \
                self.new_object.get('dot11w')
        if self.new_object.get('dot11r') is not None or self.new_object.get(
                'dot11r') is not None:
            new_object_params['dot11r'] = self.new_object.get('dot11r') or \
                self.new_object.get('dot11r')
        if self.new_object.get('splashPage') is not None or self.new_object.get(
                'splash_page') is not None:
            new_object_params['splashPage'] = self.new_object.get(
                'splashPage') or self.new_object.get('splash_page')
        if self.new_object.get('splashGuestSponsorDomains') is not None or self.new_object.get(
                'splash_guest_sponsor_domains') is not None:
            new_object_params['splashGuestSponsorDomains'] = self.new_object.get(
                'splashGuestSponsorDomains') or self.new_object.get('splash_guest_sponsor_domains')
        if self.new_object.get('oauth') is not None or self.new_object.get(
                'oauth') is not None:
            new_object_params['oauth'] = self.new_object.get('oauth') or \
                self.new_object.get('oauth')
        if self.new_object.get('localRadius') is not None or self.new_object.get(
                'local_radius') is not None:
            new_object_params['localRadius'] = self.new_object.get(
                'localRadius') or self.new_object.get('local_radius')
        if self.new_object.get('ldap') is not None or self.new_object.get(
                'ldap') is not None:
            new_object_params['ldap'] = self.new_object.get('ldap') or \
                self.new_object.get('ldap')
        if self.new_object.get('activeDirectory') is not None or self.new_object.get(
                'active_directory') is not None:
            new_object_params['activeDirectory'] = self.new_object.get(
                'activeDirectory') or self.new_object.get('active_directory')
        if self.new_object.get('radiusServers') is not None or self.new_object.get(
                'radius_servers') is not None:
            new_object_params['radiusServers'] = self.new_object.get(
                'radiusServers') or self.new_object.get('radius_servers')
        if self.new_object.get('radiusProxyEnabled') is not None or self.new_object.get(
                'radius_proxy_enabled') is not None:
            new_object_params['radiusProxyEnabled'] = self.new_object.get(
                'radiusProxyEnabled')
        if self.new_object.get('radiusTestingEnabled') is not None or self.new_object.get(
                'radius_testing_enabled') is not None:
            new_object_params['radiusTestingEnabled'] = self.new_object.get(
                'radiusTestingEnabled')
        if self.new_object.get('radiusCalledStationId') is not None or self.new_object.get(
                'radius_called_station_id') is not None:
            new_object_params['radiusCalledStationId'] = self.new_object.get(
                'radiusCalledStationId') or self.new_object.get('radius_called_station_id')
        if self.new_object.get('radiusAuthenticationNasId') is not None or self.new_object.get(
                'radius_authentication_nas_id') is not None:
            new_object_params['radiusAuthenticationNasId'] = self.new_object.get(
                'radiusAuthenticationNasId') or self.new_object.get('radius_authentication_nas_id')
        if self.new_object.get('radiusServerTimeout') is not None or self.new_object.get(
                'radius_server_timeout') is not None:
            new_object_params['radiusServerTimeout'] = self.new_object.get(
                'radiusServerTimeout') or self.new_object.get('radius_server_timeout')
        if self.new_object.get('radiusServerAttemptsLimit') is not None or self.new_object.get(
                'radius_server_attempts_limit') is not None:
            new_object_params['radiusServerAttemptsLimit'] = self.new_object.get(
                'radiusServerAttemptsLimit') or self.new_object.get('radius_server_attempts_limit')
        if self.new_object.get('radiusFallbackEnabled') is not None or self.new_object.get(
                'radius_fallback_enabled') is not None:
            new_object_params['radiusFallbackEnabled'] = self.new_object.get(
                'radiusFallbackEnabled')
        if self.new_object.get('radiusRadsec') is not None or self.new_object.get(
                'radius_radsec') is not None:
            new_object_params['radiusRadsec'] = self.new_object.get(
                'radiusRadsec') or self.new_object.get('radius_radsec')
        if self.new_object.get('radiusCoaEnabled') is not None or self.new_object.get(
                'radius_coa_enabled') is not None:
            new_object_params['radiusCoaEnabled'] = self.new_object.get(
                'radiusCoaEnabled')
        if self.new_object.get('radiusFailoverPolicy') is not None or self.new_object.get(
                'radius_failover_policy') is not None:
            new_object_params['radiusFailoverPolicy'] = self.new_object.get(
                'radiusFailoverPolicy') or self.new_object.get('radius_failover_policy')
        if self.new_object.get('radiusLoadBalancingPolicy') is not None or self.new_object.get(
                'radius_load_balancing_policy') is not None:
            new_object_params['radiusLoadBalancingPolicy'] = self.new_object.get(
                'radiusLoadBalancingPolicy') or self.new_object.get('radius_load_balancing_policy')
        if self.new_object.get('radiusAccountingEnabled') is not None or self.new_object.get(
                'radius_accounting_enabled') is not None:
            new_object_params['radiusAccountingEnabled'] = self.new_object.get(
                'radiusAccountingEnabled')
        if self.new_object.get('radiusAccountingServers') is not None or self.new_object.get(
                'radius_accounting_servers') is not None:
            new_object_params['radiusAccountingServers'] = self.new_object.get(
                'radiusAccountingServers') or self.new_object.get('radius_accounting_servers')
        if self.new_object.get('radiusAccountingInterimInterval') is not None or self.new_object.get(
                'radius_accounting_interim_interval') is not None:
            new_object_params['radiusAccountingInterimInterval'] = self.new_object.get(
                'radiusAccountingInterimInterval') or self.new_object.get('radius_accounting_interim_interval')
        if self.new_object.get('radiusAttributeForGroupPolicies') is not None or self.new_object.get(
                'radius_attribute_for_group_policies') is not None:
            new_object_params['radiusAttributeForGroupPolicies'] = self.new_object.get(
                'radiusAttributeForGroupPolicies') or self.new_object.get('radius_attribute_for_group_policies')
        if self.new_object.get('ipAssignmentMode') is not None or self.new_object.get(
                'ip_assignment_mode') is not None:
            new_object_params['ipAssignmentMode'] = self.new_object.get(
                'ipAssignmentMode') or self.new_object.get('ip_assignment_mode')
        if self.new_object.get('useVlanTagging') is not None or self.new_object.get(
                'use_vlan_tagging') is not None:
            new_object_params['useVlanTagging'] = self.new_object.get(
                'useVlanTagging')
        if self.new_object.get('concentratorNetworkId') is not None or self.new_object.get(
                'concentrator_network_id') is not None:
            new_object_params['concentratorNetworkId'] = self.new_object.get(
                'concentratorNetworkId') or self.new_object.get('concentrator_network_id')
        if self.new_object.get('secondaryConcentratorNetworkId') is not None or self.new_object.get(
                'secondary_concentrator_network_id') is not None:
            new_object_params['secondaryConcentratorNetworkId'] = self.new_object.get(
                'secondaryConcentratorNetworkId') or self.new_object.get('secondary_concentrator_network_id')
        if self.new_object.get('disassociateClientsOnVpnFailover') is not None or self.new_object.get(
                'disassociate_clients_on_vpn_failover') is not None:
            new_object_params['disassociateClientsOnVpnFailover'] = self.new_object.get(
                'disassociateClientsOnVpnFailover')
        if self.new_object.get('vlanId') is not None or self.new_object.get(
                'vlan_id') is not None:
            new_object_params['vlanId'] = self.new_object.get('vlanId') or \
                self.new_object.get('vlan_id')
        if self.new_object.get('defaultVlanId') is not None or self.new_object.get(
                'default_vlan_id') is not None:
            new_object_params['defaultVlanId'] = self.new_object.get(
                'defaultVlanId') or self.new_object.get('default_vlan_id')
        if self.new_object.get('apTagsAndVlanIds') is not None or self.new_object.get(
                'ap_tags_and_vlan_ids') is not None:
            new_object_params['apTagsAndVlanIds'] = self.new_object.get(
                'apTagsAndVlanIds') or self.new_object.get('ap_tags_and_vlan_ids')
        if self.new_object.get('walledGardenEnabled') is not None or self.new_object.get(
                'walled_garden_enabled') is not None:
            new_object_params['walledGardenEnabled'] = self.new_object.get(
                'walledGardenEnabled')
        if self.new_object.get('walledGardenRanges') is not None or self.new_object.get(
                'walled_garden_ranges') is not None:
            new_object_params['walledGardenRanges'] = self.new_object.get(
                'walledGardenRanges') or self.new_object.get('walled_garden_ranges')
        if self.new_object.get(
                'gre') is not None or self.new_object.get('gre') is not None:
            new_object_params['gre'] = self.new_object.get('gre') or \
                self.new_object.get('gre')
        if self.new_object.get('radiusOverride') is not None or self.new_object.get(
                'radius_override') is not None:
            new_object_params['radiusOverride'] = self.new_object.get(
                'radiusOverride')
        if self.new_object.get('radiusGuestVlanEnabled') is not None or self.new_object.get(
                'radius_guest_vlan_enabled') is not None:
            new_object_params['radiusGuestVlanEnabled'] = self.new_object.get(
                'radiusGuestVlanEnabled')
        if self.new_object.get('radiusGuestVlanId') is not None or self.new_object.get(
                'radius_guest_vlan_id') is not None:
            new_object_params['radiusGuestVlanId'] = self.new_object.get(
                'radiusGuestVlanId') or self.new_object.get('radius_guest_vlan_id')
        if self.new_object.get('minBitrate') is not None or self.new_object.get(
                'min_bitrate') is not None:
            new_object_params['minBitrate'] = self.new_object.get(
                'minBitrate') or self.new_object.get('min_bitrate')
        if self.new_object.get('bandSelection') is not None or self.new_object.get(
                'band_selection') is not None:
            new_object_params['bandSelection'] = self.new_object.get(
                'bandSelection') or self.new_object.get('band_selection')
        if self.new_object.get('perClientBandwidthLimitUp') is not None or self.new_object.get(
                'per_client_bandwidth_limit_up') is not None:
            new_object_params['perClientBandwidthLimitUp'] = self.new_object.get(
                'perClientBandwidthLimitUp') or self.new_object.get('per_client_bandwidth_limit_up')
        if self.new_object.get('perClientBandwidthLimitDown') is not None or self.new_object.get(
                'per_client_bandwidth_limit_down') is not None:
            new_object_params['perClientBandwidthLimitDown'] = self.new_object.get(
                'perClientBandwidthLimitDown') or self.new_object.get('per_client_bandwidth_limit_down')
        if self.new_object.get('perSsidBandwidthLimitUp') is not None or self.new_object.get(
                'per_ssid_bandwidth_limit_up') is not None:
            new_object_params['perSsidBandwidthLimitUp'] = self.new_object.get(
                'perSsidBandwidthLimitUp') or self.new_object.get('per_ssid_bandwidth_limit_up')
        if self.new_object.get('perSsidBandwidthLimitDown') is not None or self.new_object.get(
                'per_ssid_bandwidth_limit_down') is not None:
            new_object_params['perSsidBandwidthLimitDown'] = self.new_object.get(
                'perSsidBandwidthLimitDown') or self.new_object.get('per_ssid_bandwidth_limit_down')
        if self.new_object.get('lanIsolationEnabled') is not None or self.new_object.get(
                'lan_isolation_enabled') is not None:
            new_object_params['lanIsolationEnabled'] = self.new_object.get(
                'lanIsolationEnabled')
        if self.new_object.get('visible') is not None or self.new_object.get(
                'visible') is not None:
            new_object_params['visible'] = self.new_object.get('visible')
        if self.new_object.get('availableOnAllAps') is not None or self.new_object.get(
                'available_on_all_aps') is not None:
            new_object_params['availableOnAllAps'] = self.new_object.get(
                'availableOnAllAps')
        if self.new_object.get('availabilityTags') is not None or self.new_object.get(
                'availability_tags') is not None:
            new_object_params['availabilityTags'] = self.new_object.get(
                'availabilityTags') or self.new_object.get('availability_tags')
        if self.new_object.get('adaptivePolicyGroupId') is not None or self.new_object.get(
                'adaptive_policy_group_id') is not None:
            new_object_params['adaptivePolicyGroupId'] = self.new_object.get(
                'adaptivePolicyGroupId') or self.new_object.get('adaptive_policy_group_id')
        if self.new_object.get('mandatoryDhcpEnabled') is not None or self.new_object.get(
                'mandatory_dhcp_enabled') is not None:
            new_object_params['mandatoryDhcpEnabled'] = self.new_object.get(
                'mandatoryDhcpEnabled')
        if self.new_object.get('adultContentFilteringEnabled') is not None or self.new_object.get(
                'adult_content_filtering_enabled') is not None:
            new_object_params['adultContentFilteringEnabled'] = self.new_object.get(
                'adultContentFilteringEnabled')
        if self.new_object.get('dnsRewrite') is not None or self.new_object.get(
                'dns_rewrite') is not None:
            new_object_params['dnsRewrite'] = self.new_object.get(
                'dnsRewrite') or self.new_object.get('dns_rewrite')
        if self.new_object.get('speedBurst') is not None or self.new_object.get(
                'speed_burst') is not None:
            new_object_params['speedBurst'] = self.new_object.get(
                'speedBurst') or self.new_object.get('speed_burst')
        if self.new_object.get('namedVlans') is not None or self.new_object.get(
                'named_vlans') is not None:
            new_object_params['namedVlans'] = self.new_object.get(
                'namedVlans') or self.new_object.get('named_vlans')
        if self.new_object.get('localAuthFallback') is not None or self.new_object.get(
                'local_auth_fallback') is not None:
            new_object_params['localAuthFallback'] = self.new_object.get(
                'localAuthFallback') or self.new_object.get('local_auth_fallback')
        if self.new_object.get('radiusAccountingStartDelay') is not None or self.new_object.get(
                'radius_accounting_start_delay') is not None:
            new_object_params['radiusAccountingStartDelay'] = self.new_object.get(
                'radiusAccountingStartDelay') or self.new_object.get('radius_accounting_start_delay')
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        if self.new_object.get('number') is not None or self.new_object.get(
                'number') is not None:
            new_object_params['number'] = self.new_object.get('number') or \
                self.new_object.get('number')
        return new_object_params

    def get_object_by_name(self, name):
        result = None
        # NOTE: Does not have a get by name method, using get all
        try:
            items = self.meraki.exec_meraki(
                family="wireless",
                function="getNetworkWirelessSsids",
                params=self.get_all_params(name=name),
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'name', name)
            if result is None:
                result = items
        except Exception as e:
            print("Error: ", e)
            result = None
        return result

    def get_object_by_id(self, id):
        result = None
        try:
            items = self.meraki.exec_meraki(
                family="wireless",
                function="getNetworkWirelessSsid",
                params=self.get_params_by_id()
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = items
        except Exception as e:
            print("Error: ", e)
            result = None
        return result

    def exists(self):
        prev_obj = None
        id_exists = False
        name_exists = False
        o_id = self.new_object.get(
            "networkId") or self.new_object.get("network_id")
        o_id = o_id or self.new_object.get(
            "number") or self.new_object.get("number")
        name = o_id or self.new_object.get("name")
        if o_id:
            prev_obj = self.get_object_by_id(o_id)
            id_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if not id_exists and name:
            prev_obj = self.get_object_by_name(name)
            name_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if name_exists:
            _id = prev_obj.get("id")
            _id = _id or prev_obj.get("number")
            if id_exists and name_exists and o_id != _id:
                raise InconsistentParameters(
                    "The 'id' and 'name' params don't refer to the same object")
            if _id:
                self.new_object.update(dict(id=_id))
                self.new_object.update(dict(number=_id))
            if _id:
                prev_obj = self.get_object_by_id(_id)
        it_exists = prev_obj is not None and isinstance(prev_obj, dict)
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("name", "name"),
            ("enabled", "enabled"),
            ("authMode", "authMode"),
            ("enterpriseAdminAccess", "enterpriseAdminAccess"),
            ("ssidAdminAccessible", "ssidAdminAccessible"),
            ("encryptionMode", "encryptionMode"),
            ("psk", "psk"),
            ("wpaEncryptionMode", "wpaEncryptionMode"),
            ("dot11w", "dot11w"),
            ("dot11r", "dot11r"),
            ("splashPage", "splashPage"),
            ("splashGuestSponsorDomains", "splashGuestSponsorDomains"),
            ("oauth", "oauth"),
            ("localRadius", "localRadius"),
            ("ldap", "ldap"),
            ("activeDirectory", "activeDirectory"),
            ("radiusServers", "radiusServers"),
            ("radiusProxyEnabled", "radiusProxyEnabled"),
            ("radiusTestingEnabled", "radiusTestingEnabled"),
            ("radiusCalledStationId", "radiusCalledStationId"),
            ("radiusAuthenticationNasId", "radiusAuthenticationNasId"),
            ("radiusServerTimeout", "radiusServerTimeout"),
            ("radiusServerAttemptsLimit", "radiusServerAttemptsLimit"),
            ("radiusFallbackEnabled", "radiusFallbackEnabled"),
            ("radiusRadsec", "radiusRadsec"),
            ("radiusCoaEnabled", "radiusCoaEnabled"),
            ("radiusFailoverPolicy", "radiusFailoverPolicy"),
            ("radiusLoadBalancingPolicy", "radiusLoadBalancingPolicy"),
            ("radiusAccountingEnabled", "radiusAccountingEnabled"),
            ("radiusAccountingServers", "radiusAccountingServers"),
            ("radiusAccountingInterimInterval", "radiusAccountingInterimInterval"),
            ("radiusAttributeForGroupPolicies", "radiusAttributeForGroupPolicies"),
            ("ipAssignmentMode", "ipAssignmentMode"),
            ("useVlanTagging", "useVlanTagging"),
            ("concentratorNetworkId", "concentratorNetworkId"),
            ("secondaryConcentratorNetworkId", "secondaryConcentratorNetworkId"),
            ("disassociateClientsOnVpnFailover", "disassociateClientsOnVpnFailover"),
            ("vlanId", "vlanId"),
            ("defaultVlanId", "defaultVlanId"),
            ("apTagsAndVlanIds", "apTagsAndVlanIds"),
            ("walledGardenEnabled", "walledGardenEnabled"),
            ("walledGardenRanges", "walledGardenRanges"),
            ("gre", "gre"),
            ("radiusOverride", "radiusOverride"),
            ("radiusGuestVlanEnabled", "radiusGuestVlanEnabled"),
            ("radiusGuestVlanId", "radiusGuestVlanId"),
            ("minBitrate", "minBitrate"),
            ("bandSelection", "bandSelection"),
            ("perClientBandwidthLimitUp", "perClientBandwidthLimitUp"),
            ("perClientBandwidthLimitDown", "perClientBandwidthLimitDown"),
            ("perSsidBandwidthLimitUp", "perSsidBandwidthLimitUp"),
            ("perSsidBandwidthLimitDown", "perSsidBandwidthLimitDown"),
            ("lanIsolationEnabled", "lanIsolationEnabled"),
            ("visible", "visible"),
            ("availableOnAllAps", "availableOnAllAps"),
            ("availabilityTags", "availabilityTags"),
            ("adaptivePolicyGroupId", "adaptivePolicyGroupId"),
            ("mandatoryDhcpEnabled", "mandatoryDhcpEnabled"),
            ("adultContentFilteringEnabled", "adultContentFilteringEnabled"),
            ("dnsRewrite", "dnsRewrite"),
            ("speedBurst", "speedBurst"),
            ("namedVlans", "namedVlans"),
            ("localAuthFallback", "localAuthFallback"),
            ("radiusAccountingStartDelay", "radiusAccountingStartDelay"), ("number", "number"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (ISE) params
        # If any does not have eq params, it requires update
        current_obj["number"] = str(current_obj.get("number"))
        return any(
            not meraki_compare_equality2(
                current_obj.get(meraki_param),
                requested_obj.get(ansible_param)) for (
                meraki_param,
                ansible_param) in obj_params)

    def update(self):
        id = self.new_object.get("id")
        id = id or self.new_object.get("number")
        name = self.new_object.get("name")
        result = None
        if not id:
            prev_obj_name = self.get_object_by_name(name)
            id_ = None
            if prev_obj_name:
                id_ = prev_obj_name.get("id")
                id_ = id_ or prev_obj_name.get("number")
            if id_:
                self.new_object.update(dict(number=id_))
        result = self.meraki.exec_meraki(
            family="wireless",
            function="updateNetworkWirelessSsid",
            params=self.update_by_id_params(),
            op_modifies=True,
        )
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail(
                "ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        meraki = MERAKI(self._task.args)
        obj = NetworksWirelessSsids(self._task.args, meraki)

        state = self._task.args.get("state")

        response = None
        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    response = obj.update()
                    meraki.object_updated()
                else:
                    response = prev_obj
                    meraki.object_already_present()
            else:
                meraki.fail_json(
                    "Object does not exists, plugin only has update")

        self._result.update(dict(meraki_response=response))
        self._result.update(meraki.exit_json())
        return self._result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_appliance_rf_profiles
short_description: Resource module for networks _appliance _rf _profiles
description:
  - Manage operations create, update and delete of the resource networks _appliance
    _rf _profiles.
  - Creates new RF profile for this network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  fiveGhzSettings:
    description: Settings related to 5Ghz band.
    suboptions:
      axEnabled:
        description: Determines whether ax radio on 5GHz band is on or off. Can be
          either true or false. If false, we highly recommend disabling band steering.
          Defaults to true.
        type: bool
      minBitrate:
        description: Sets min bitrate (Mbps) of 5Ghz band. Can be one of '6', '9',
          '12', '18', '24', '36', '48' or '54'. Defaults to 12.
        type: int
    type: dict
  name:
    description: The name of the new profile. Must be unique. This param is required
      on creation.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  perSsidSettings:
    description: Per-SSID radio settings by number.
    suboptions:
      '1':
        description: Settings for SSID 1.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz', '5ghz', '6ghz' or 'multi'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
        type: dict
      '2':
        description: Settings for SSID 2.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz', '5ghz', '6ghz' or 'multi'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
        type: dict
      '3':
        description: Settings for SSID 3.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz', '5ghz', '6ghz' or 'multi'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
        type: dict
      '4':
        description: Settings for SSID 4.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz', '5ghz', '6ghz' or 'multi'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
        type: dict
    type: dict
  twoFourGhzSettings:
    description: Settings related to 2.4Ghz band.
    suboptions:
      axEnabled:
        description: Determines whether ax radio on 2.4Ghz band is on or off. Can
          be either true or false. If false, we highly recommend disabling band steering.
          Defaults to true.
        type: bool
      minBitrate:
        description: Sets min bitrate (Mbps) of 2.4Ghz band. Can be one of '1', '2',
          '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'. Defaults to
          11.
        type: float
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance createNetworkApplianceRfProfile
    description: Complete reference of the createNetworkApplianceRfProfile API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-appliance-rf-profile
notes:
  - SDK Method used are
    appliance.Appliance.create_network_appliance_rf_profile,
  - Paths used are
    post /networks/{networkId}/appliance/rfProfiles,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_appliance_rf_profiles:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    fiveGhzSettings:
      axEnabled: true
      minBitrate: 48
    name: MX RF Profile
    networkId: string
    perSsidSettings:
      '1':
        bandOperationMode: dual
        bandSteeringEnabled: true
      '2':
        bandOperationMode: dual
        bandSteeringEnabled: true
      '3':
        bandOperationMode: dual
        bandSteeringEnabled: true
      '4':
        bandOperationMode: dual
        bandSteeringEnabled: true
    twoFourGhzSettings:
      axEnabled: true
      minBitrate: 12.0
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "networkId": "string",
      "name": "string",
      "twoFourGhzSettings": {
        "minBitrate": 0,
        "axEnabled": true
      },
      "fiveGhzSettings": {
        "minBitrate": 0,
        "axEnabled": true
      },
      "perSsidSettings": {
        "1": {
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "2": {
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "3": {
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "4": {
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        }
      }
    }
"""



#!/usr/bin/python

# Copyright (c) 2016, Thierno IB. BARRY @barryib
# Sponsored by Polyconseil http://polyconseil.fr.
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: kibana_plugin
short_description: Manage Kibana plugins
description:
  - This module can be used to manage Kibana plugins.
author: Thierno IB. BARRY (@barryib)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - Name of the plugin to install.
    required: true
    type: str
  state:
    description:
      - Desired state of a plugin.
    choices: ["present", "absent"]
    default: present
    type: str
  url:
    description:
      - Set exact URL to download the plugin from.
      - For local file, prefix its absolute path with C(file://).
    type: str
  timeout:
    description:
      - 'Timeout setting: V(30s), V(1m), V(1h) and so on.'
    default: 1m
    type: str
  plugin_bin:
    description:
      - Location of the Kibana binary.
    default: /opt/kibana/bin/kibana
    type: path
  plugin_dir:
    description:
      - Your configured plugin directory specified in Kibana.
    default: /opt/kibana/installedPlugins/
    type: path
  version:
    description:
      - Version of the plugin to be installed.
      - If the plugin is installed with in a previous version, it is B(not) updated unless O(force=true).
    type: str
  force:
    description:
      - Delete and re-install the plugin. It can be useful for plugins update.
    type: bool
    default: false
  allow_root:
    description:
      - Whether to allow C(kibana) and C(kibana-plugin) to be run as root. Passes the C(--allow-root) flag to these commands.
    type: bool
    default: false
    version_added: 2.3.0
"""

EXAMPLES = r"""
- name: Install Elasticsearch head plugin
  community.general.kibana_plugin:
    state: present
    name: elasticsearch/marvel

- name: Install specific version of a plugin
  community.general.kibana_plugin:
    state: present
    name: elasticsearch/marvel
    version: '2.3.3'

- name: Uninstall Elasticsearch head plugin
  community.general.kibana_plugin:
    state: absent
    name: elasticsearch/marvel
"""

RETURN = r"""
cmd:
  description: The launched command during plugin management (install / remove).
  returned: success
  type: str
name:
  description: The plugin name to install or remove.
  returned: success
  type: str
url:
  description: The URL from where the plugin is installed from.
  returned: success
  type: str
timeout:
  description: The timeout for plugin download.
  returned: success
  type: str
state:
  description: The state for the managed plugin.
  returned: success
  type: str
"""

import os

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._version import LooseVersion

PACKAGE_STATE_MAP = dict(present="--install", absent="--remove")


def parse_plugin_repo(string):
    elements = string.split("/")

    # We first consider the simplest form: pluginname
    repo = elements[0]

    # We consider the form: username/pluginname
    if len(elements) > 1:
        repo = elements[1]

    # remove elasticsearch- prefix
    # remove es- prefix
    for string in ("elasticsearch-", "es-"):
        if repo.startswith(string):
            return repo[len(string) :]

    return repo


def is_plugin_present(plugin_dir, working_dir):
    return os.path.isdir(os.path.join(working_dir, plugin_dir))


def parse_error(string):
    reason = "reason: "
    try:
        return string[string.index(reason) + len(reason) :].strip()
    except ValueError:
        return string


def install_plugin(module, plugin_bin, plugin_name, url, timeout, allow_root, kibana_version="4.6"):
    if LooseVersion(kibana_version) > LooseVersion("4.6"):
        kibana_plugin_bin = os.path.join(os.path.dirname(plugin_bin), "kibana-plugin")
        cmd_args = [kibana_plugin_bin, "install"]
        if url:
            cmd_args.append(url)
        else:
            cmd_args.append(plugin_name)
    else:
        cmd_args = [plugin_bin, "plugin", PACKAGE_STATE_MAP["present"], plugin_name]

        if url:
            cmd_args.extend(["--url", url])

    if timeout:
        cmd_args.extend(["--timeout", timeout])

    if allow_root:
        cmd_args.append("--allow-root")

    if module.check_mode:
        return True, " ".join(cmd_args), "check mode", ""

    rc, out, err = module.run_command(cmd_args)
    if rc != 0:
        reason = parse_error(out)
        module.fail_json(msg=reason)

    return True, " ".join(cmd_args), out, err


def remove_plugin(module, plugin_bin, plugin_name, allow_root, kibana_version="4.6"):
    if LooseVersion(kibana_version) > LooseVersion("4.6"):
        kibana_plugin_bin = os.path.join(os.path.dirname(plugin_bin), "kibana-plugin")
        cmd_args = [kibana_plugin_bin, "remove", plugin_name]
    else:
        cmd_args = [plugin_bin, "plugin", PACKAGE_STATE_MAP["absent"], plugin_name]

    if allow_root:
        cmd_args.append("--allow-root")

    if module.check_mode:
        return True, " ".join(cmd_args), "check mode", ""

    rc, out, err = module.run_command(cmd_args)
    if rc != 0:
        reason = parse_error(out)
        module.fail_json(msg=reason)

    return True, " ".join(cmd_args), out, err


def get_kibana_version(module, plugin_bin, allow_root):
    cmd_args = [plugin_bin, "--version"]

    if allow_root:
        cmd_args.append("--allow-root")

    rc, out, err = module.run_command(cmd_args)
    if rc != 0:
        module.fail_json(msg=f"Failed to get Kibana version : {err}")

    return out.strip()


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=True),
            state=dict(default="present", choices=list(PACKAGE_STATE_MAP.keys())),
            url=dict(),
            timeout=dict(default="1m"),
            plugin_bin=dict(default="/opt/kibana/bin/kibana", type="path"),
            plugin_dir=dict(default="/opt/kibana/installedPlugins/", type="path"),
            version=dict(),
            force=dict(default=False, type="bool"),
            allow_root=dict(default=False, type="bool"),
        ),
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    name = module.params["name"]
    state = module.params["state"]
    url = module.params["url"]
    timeout = module.params["timeout"]
    plugin_bin = module.params["plugin_bin"]
    plugin_dir = module.params["plugin_dir"]
    version = module.params["version"]
    force = module.params["force"]
    allow_root = module.params["allow_root"]

    changed, cmd, out, err = False, "", "", ""

    kibana_version = get_kibana_version(module, plugin_bin, allow_root)

    present = is_plugin_present(parse_plugin_repo(name), plugin_dir)

    # skip if the state is correct
    if (present and state == "present" and not force) or (state == "absent" and not present and not force):
        module.exit_json(changed=False, name=name, state=state)

    if version:
        name = f"{name}/{version}"

    if state == "present":
        if force:
            remove_plugin(module, plugin_bin, name, allow_root, kibana_version)
        changed, cmd, out, err = install_plugin(module, plugin_bin, name, url, timeout, allow_root, kibana_version)

    elif state == "absent":
        changed, cmd, out, err = remove_plugin(module, plugin_bin, name, allow_root, kibana_version)

    module.exit_json(changed=changed, cmd=cmd, name=name, state=state, url=url, timeout=timeout, stdout=out, stderr=err)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for iosxr_bgp_neighbor_address_family
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: iosxr_bgp_neighbor_address_family
short_description: Resource module to configure BGP Neighbor Address family.
description:
- This module configures and manages the attributes of BGP global on Cisco IOS-XR platforms.
version_added: 2.0.0
author: Ashwini Mhatre (@amhatre)
notes:
- This module works with connection C(network_cli).
options:
    config:
      description: BGP neighbor address family configurations.
      type: dict
      suboptions:
        as_number:
          description: Autonomous system number.
          type: str
        neighbors:
            description: A list of BGP neighbor address family configurations.
            type: list
            elements: dict
            suboptions:
              neighbor_address:
                description:
                  - Neighbor router address.
                type: str
                required: true
              address_family:
                description: Enable address family and enter its config mode
                type: list
                elements: dict
                suboptions:
                  afi:
                    description: address family.
                    type: str
                    choices: [ 'ipv4', 'ipv6', 'l2vpn', 'link-state', 'vpnv4', 'vpnv6']
                  safi:
                    description: Address Family modifier
                    type: str
                    choices: [ 'flowspec', 'mdt', 'multicast', 'mvpn', 'rt-filter', 'tunnel', 'unicast', 'labeled-unicast' ]
                  aigp: &aigp
                    description: AIGP attribute
                    type: dict
                    suboptions:
                      disable:
                        description: Ignore AIGP attribute.
                        type: bool
                      set:
                        description: Set AIGP attribute.
                        type: bool
                      send_cost_community_disable:
                        description: send AIGP attribute.
                        type: bool
                      send_med:
                        description: send med options.
                        type: dict
                        suboptions:
                          set:
                            type: bool
                            description: set Send AIGP value in MED.
                          disable:
                            description: disable Send AIGP value in MED.
                            type: bool
                  allowas_in: &allowas_in
                    type: dict
                    description: Allow as-path with my AS present in it.
                    suboptions:
                      value:
                        type: int
                        description: Number of occurences of AS number 1-10.
                      set:
                        type: bool
                        description: set allowas_in
                  as_override: &as_override
                    type: dict
                    description: Override matching AS-number while sending update
                    suboptions:
                      set:
                        type: bool
                        description: set as_override
                      inheritance_disable:
                        type: bool
                        description: Prevent as-override from being inherited from the parent.
                  bestpath_origin_as_allow_invalid:
                    type: bool
                    description: Change default route selection criteria.Allow BGP origin-AS knobs.
                  capability_orf_prefix: &capability
                    type: str
                    description: Advertise address prefix ORF capability to this neighbor.
                    choices: ['both', 'send', 'none', 'receive']
                  default_originate: &default_originate
                    type: dict
                    description: Originate default route to this neighbor.
                    suboptions:
                      set:
                        type: bool
                        description: set default route.
                      route_policy:
                        type: str
                        description: Route policy to specify criteria to originate default
                      inheritance_disable:
                        type: bool
                        description: Prevent default-originate from being inherited from the parent.
                  long_lived_graceful_restart: &long_lived_graceful_restart
                    type: dict
                    description: Enable long lived graceful restart support.
                    suboptions:
                      capable:
                        type: bool
                        description: Treat neighbor as LLGR capable.
                      stale_time:
                        type: dict
                        description: Maximum time to wait before purging long-lived stale routes.
                        suboptions:
                          send:
                            type: int
                            description: max send time
                          accept:
                            type: int
                            description: max accept time
                  maximum_prefix: &maximum_prefix
                    type: dict
                    description: Maximum number of prefixes to accept from this peer.
                    suboptions:
                      max_limit:
                        type: int
                        description:  maximum no. of prefix limit.<1-4294967295.
                      threshold_value:
                        type: int
                        description: hreshold value (%) at which to generate a warning msg <1-100>.
                      restart:
                        type: int
                        description: Restart time interval.
                      warning_only:
                        type: bool
                        description: Only give warning message when limit is exceeded.
                      discard_extra_paths:
                        description: Discard extra paths when limit is exceeded.
                        type: bool
                  multipath: &multipath
                    type: bool
                    description: Paths from this neighbor is eligible for multipath.
                  next_hop_self: &next_hop_self
                    type: dict
                    description: Disable the next hop calculation for this neighbor.
                    suboptions:
                      set:
                        type: bool
                        description: set next hop self.
                      inheritance_disable:
                        type: bool
                        description: Prevent next_hop_self from being inherited from the parent.
                  next_hop_unchanged: &next_hop_unchanged
                    type: dict
                    description: Disable the next hop calculation for this neighbor.
                    suboptions:
                      set:
                        type: bool
                        description: set next hop unchanged.
                      inheritance_disable:
                        type: bool
                        description: Prevent next_hop_unchanged from being inherited from the parent.
                      multipath:
                        type: bool
                        description: Do not overwrite nexthop before advertising multipaths.
                  optimal_route_reflection_group_name: &optimal_route_reflection
                    type: str
                    description: Configure optimal-route-reflection group.
                  orf_route_policy: &orf_rp
                    type: str
                    description: Specify ORF and inbound filtering criteria.'
                  origin_as:
                    description: BGP origin-AS knobs.
                    type: dict
                    suboptions:
                      validation:
                        description: BGP origin-AS validation knobs.
                        type: dict
                        suboptions:
                          disable:
                            description: Disable RPKI origin-AS validation.
                            type: bool
                  remove_private_AS: &remove_private_AS
                    type: dict
                    description: Remove private AS number from outbound updates.
                    suboptions:
                      set:
                        type: bool
                        description: set remove private As.
                      inbound:
                        type: bool
                        description: Remove private AS number from inbound updates.
                      entire_aspath:
                        type: bool
                        description: remove only if all ASes in the path are private.
                      inheritance_disable:
                        type: bool
                        description: Prevent remove-private-AS from being inherited from the parent.
                  route_policy: &route_policy
                    type: dict
                    description: Apply route policy to neighbor.
                    suboptions:
                      inbound:
                        type: str
                        description: Apply route policy to inbound routes.
                      outbound:
                        type: str
                        description: Apply route policy to outbound routes.
                  route_reflector_client: &route_reflector_client
                    type: dict
                    description:  Configure a neighbor as Route Reflector client.
                    suboptions:
                      set:
                        type: bool
                        description: set route-reflector-client.
                      inheritance_disable:
                        type: bool
                        description: Prevent route-reflector-client from being inherited from the parent.
                  send_community_ebgp: &send_community_ebgp
                    description: Send community attribute to this external neighbor.
                    type: dict
                    suboptions:
                      set:
                        type: bool
                        description: set send_community_ebgp.
                      inheritance_disable:
                        type: bool
                        description: Prevent send_community_ebgp from being inherited from the parent.
                  send_community_gshut_ebgp: &send_community_gshut_ebgp
                    description: Allow the g-shut community to be sent to this external neighbor.
                    type: dict
                    suboptions:
                      set:
                        type: bool
                        description: set send_community_gshut_ebgp.
                      inheritance_disable:
                        type: bool
                        description: Prevent send_community_gshut_ebgp from being inherited from the parent.
                  send_extended_community_ebgp: &send_extended_community_ebgp
                    description: Send extended community attribute to this external neighbor.
                    type: dict
                    suboptions:
                      set:
                        type: bool
                        description: set send_extended_community_ebgp.
                      inheritance_disable:
                        type: bool
                        description: Prevent send_extended_community_ebgp from being inherited from the parent.
                  send_multicast_attributes:
                    description: Send multicast attributes to this neighbor .
                    type: dict
                    suboptions:
                      set:
                        type: bool
                        description: set send_multicast_attributes.
                      disable:
                        type: bool
                        description: Disable send multicast attributes.
                  soft_reconfiguration: &soft_reconfiguration
                    description: Per neighbor soft reconfiguration.
                    type: dict
                    suboptions:
                      inbound:
                        type: dict
                        description: inbound soft reconfiguration
                        suboptions:
                          set:
                            type: bool
                            description: set inbound
                          always:
                            type: bool
                            description: Allow inbound soft reconfiguration for this neighbor. Always use soft reconfig, even if route refresh is supported.
                          inheritance_disable:
                            type: bool
                            description: Prevent soft_reconfiguration from being inherited from the parent.
                  weight: &wt
                    type: int
                    description: Set default weight for routes from this neighbor.
                  validation: &validation
                    type: dict
                    description: Flowspec Validation for this neighbor.
                    suboptions:
                      set:
                       type: bool
                       description: set validation.
                      redirect:
                        type: bool
                        description: Flowspec Redirect nexthop Validation.
                      disable:
                        type: bool
                        description:  disable validation.
        vrfs:
          description: Configure BGP neighbor afin a VRF.
          type: list
          elements: dict
          suboptions:
            vrf:
              description: VRF name.
              type: str
            neighbors:
                description: A list of BGP neighbor address family configurations.
                type: list
                elements: dict
                suboptions:
                  neighbor_address:
                    description:
                      - Neighbor router address.
                    type: str
                    required: true
                  address_family:
                    description: Enable address family and enter its config mode
                    type: list
                    elements: dict
                    suboptions:
                      afi:
                        description: address family.
                        type: str
                        choices: [ 'ipv4', 'ipv6']
                      safi:
                        description: Address Family modifier
                        type: str
                        choices: [ 'flowspec', 'multicast', 'mvpn', 'unicast', 'labeled-unicast' ]
                      aigp: *aigp
                      allowas_in: *allowas_in
                      as_override:
                        type: dict
                        description: Override matching AS-number while sending update
                        aliases:
                          - as_overrride
                        suboptions:
                          set:
                            type: bool
                            description: set as_override
                          inheritance_disable:
                            type: bool
                            description: Prevent as-override from being inherited from the parent.
                      capability_orf_prefix: *capability
                      default_originate: *default_originate
                      long_lived_graceful_restart: *long_lived_graceful_restart
                      maximum_prefix: *maximum_prefix
                      multipath: *multipath
                      next_hop_self: *next_hop_self
                      next_hop_unchanged: *next_hop_unchanged
                      optimal_route_reflection_group_name: *optimal_route_reflection
                      orf_route_policy: *orf_rp
                      remove_private_AS: *remove_private_AS
                      route_policy: *route_policy
                      route_reflector_client: *route_reflector_client
                      send_community_ebgp: *send_community_ebgp
                      send_community_gshut_ebgp: *send_community_gshut_ebgp
                      send_extended_community_ebgp: *send_extended_community_ebgp
                      soft_reconfiguration: *soft_reconfiguration
                      site_of_origin:
                        description: Site-of-Origin extended community associated with the neighbor.
                        type: str
                      weight: *wt
                      validation: *validation

    running_config:
      description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the Iosxr device by
        executing the command B(show running-config router bgp).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
      type: str
    state:
      description:
      - The state the configuration should be left in.
      type: str
      choices: [deleted, merged, overridden, replaced, gathered, rendered, parsed]
      default: merged
"""
EXAMPLES = """
# Using merged
# Before state:
# -------------
# RP/0/0/CPU0:iosxr-02#show running-config router bgp
# Sat Feb 20 03:49:43.618 UTC
# router bgp 65536
#  bgp router-id 192.0.2.1
#  address-family ipv4 unicast
#  address-family vpnv4 unicast
#  neighbor 192.0.2.2
#   remote-as 65537
#  neighbor 192.0.2.3
#   remote-as 65538
#  vrf vrf1
#   rd auto
#   neighbor 192.0.2.4
#    remote-as 65539
#  vrf vrf2
#   rd auto
#   neighbor 192.0.2.5
#    remote-as 65540

- name: Merge the provided configuration with the existing running configuration
  cisco.iosxr.iosxr_bgp_neighbor_address_family:
    state: merged
    config:
      as_number: 65536
      neighbors:
        - neighbor_address: 192.0.2.2
          address_family:
            - afi: ipv4
              safi: unicast
              multipath: true
              default_originate:
                set: true
              weight: 5
        - neighbor_address: 192.0.2.3
          address_family:
            - afi: ipv4
              safi: unicast
              multipath: true
              default_originate:
                set: true
              weight: 4
      vrfs:
        - vrf: vrf1
          neighbors:
            - neighbor_address: 192.0.2.4
              address_family:
                - afi: ipv4
                  safi: unicast
                  multipath: true
                  default_originate:
                    set: true
                  capability_orf_prefix: both
        - vrf: vrf2
          neighbors:
            - neighbor_address: 192.0.2.5
              address_family:
                - afi: ipv4
                  safi: unicast
                  multipath: true
                  default_originate:
                    set: true
                  capability_orf_prefix: both
# Task output
# -------------
# commands:
# - router bgp 65536
# - neighbor 192.0.2.2
# - address-family ipv4 unicast
# - default-originate
# - multipath
# - weight 5
# - neighbor 192.0.2.3
# - address-family ipv4 unicast
# - default-originate
# - multipath
# - weight 4
# - vrf vrf1
# - neighbor 192.0.2.4
# - address-family ipv4 unicast
# - capability orf prefix both
# - default-originate
# - multipath
# - vrf vrf2
# - neighbor 192.0.2.5
# - address-family ipv4 unicast
# - capability orf prefix both
# - default-originate
# - multipath
#
#
# after:
#   as_number: 65536
#   neighbors:
#     - neighbor_address: 192.0.2.2
#       address_family:
#         - afi: "ipv4"
#           safi: "unicast"
#           multipath: true
#           default_originate:
#             set: true
#           weight: 5
#     - neighbor_address: 192.0.2.3
#       address_family:
#         - afi: "ipv4"
#           safi: "unicast"
#           multipath: true
#           default_originate:
#             set: true
#           weight: 4
#   vrfs:
#     - vrf: vrf1
#       neighbors:
#         - neighbor_address: 192.0.2.4
#           address_family:
#             - afi: "ipv4"
#               safi: "unicast"
#               multipath: true
#               default_originate:
#                 set: true
#               capability_orf_prefix: both
#     - vrf: vrf2
#       neighbors:
#         - neighbor_address: 192.0.2.5
#           address_family:
#             - afi: "ipv4"
#               safi: "unicast"
#               multipath: true
#               default_originate:
#                 set: true
#               capability_orf_prefix: both
#
#
# After state:
# -------------
# RP/0/0/CPU0:iosxr-02#show running-config router bgp
# Sat Feb 20 03:49:43.618 UTC
# router bgp 65536
#  bgp router-id 192.0.1.1
#  address-family ipv4 unicast
#  address-family vpnv4 unicast
#  neighbor 192.0.2.2
#   remote-as 65537
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  neighbor 1.1.1.2
#   remote-as 65538
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  vrf vrf1
#   rd auto
#   neighbor 192.0.2.4
#    remote-as 65539
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate
#  vrf vrf2
#   rd auto
#   neighbor 192.0.2.5
#    remote-as 65540
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate
#
#
# Using delete
# Before state:
# -------------
#
# RP/0/0/CPU0:iosxr-02#show running-config router bgp
# Sat Feb 20 03:49:43.618 UTC
# router bgp 65536
#  bgp router-id 192.0.1.1
#  address-family ipv4 unicast
#  address-family vpnv4 unicast
#  neighbor 192.0.2.2
#   remote-as 65537
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  neighbor 192.0.2.3
#   remote-as 65538
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  vrf vrf1
#   rd auto
#   neighbor 192.0.2.4
#    remote-as 65539
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate
#  vrf vrf2
#   rd auto
#   neighbor 192.0.2.5
#    remote-as 65540
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate

- name: Delete the provided configuration
  cisco.iosxr.iosxr_bgp_neighbor_address_family:
    state: deleted
    config:
      as_number: 65536
      neighbors:
        - neighbor_address: 192.0.2.2
          address_family:
            - afi: ipv4
              safi: unicast
              multipath: true
              default_originate:
                set: true
              weight: 5

# Task output
# -------------
#
# commands:
# - router bgp 65536
# - neighbor 192.0.2.2
# - no address-family ipv4 unicast
#
#
# after:
#   as_number: 65536
#   neighbors:
#     - neighbor_address: 192.0.2.3
#       address_family:
#         - afi: "ipv4"
#           safi: "unicast"
#           multipath: true
#           default_originate:
#             set: true
#           weight: 4
#   vrfs:
#     - vrf: vrf1
#       neighbors:
#         - neighbor_address: 192.0.2.4
#           address_family:
#             - afi: "ipv4"
#               safi: "unicast"
#               multipath: true
#               default_originate:
#                 set: true
#               capability_orf_prefix: both
#         - neighbor_address: 192.0.2.5
#           address_family:
#             - afi: "ipv4"
#               safi: "unicast"
#               multipath: true
#               default_originate:
#                 set: true
#               capability_orf_prefix: both
#
#
# Using Replaced
# Before state:
# -------------
#
# RP/0/0/CPU0:iosxr-02#show running-config router bgp
# Sat Feb 20 03:49:43.618 UTC
# router bgp 65536
#  bgp router-id 192.0.1.1
#  address-family ipv4 unicast
#  address-family vpnv4 unicast
#  neighbor 192.0.2.2
#   remote-as 65537
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  neighbor 192.0.2.3
#   remote-as 65538
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  vrf vrf1
#   rd auto
#   neighbor 192.0.2.4
#    remote-as 65539
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate
#  vrf vrf2
#   rd auto
#   neighbor 192.0.2.5
#    remote-as 65540
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate

- name: Replace the provided configuration with the existing running configuration
  cisco.iosxr.iosxr_bgp_neighbor_address_family:
    state: replaced
    config:
      as_number: 65536
      neighbors:
        - neighbor_address: 192.0.2.2
          address_family:
            - afi: ipv4
              safi: unicast
              default_originate:
                set: true
              weight: 4
# Task output
# -------------
# commands:
# - router bgp 65536
# - neighbor 192.0.2.2
# - address-family ipv4 unicast
# - no multipath
# - weight 4
#
# after:
#   as_number: 65536
#   neighbors:
#     - neighbor_address: 192.0.2.2
#       address_family:
#         - afi: "ipv4"
#           safi: "unicast"
#           multipath: true
#           default_originate:
#             set: true
#           weight: 4
#     - neighbor_address: 192.0.2.3
#       address_family:
#         - afi: "ipv4"
#           safi: "unicast"
#           multipath: true
#           default_originate:
#             set: true
#           weight: 5
#   vrfs:
#     - vrf: vrf1
#       neighbors:
#         - neighbor_address: 192.0.2.4
#           address_family:
#             - afi: "ipv4"
#               safi: "unicast"
#               multipath: true
#               default_originate:
#                 set: true
#               capability_orf_prefix: both
#         - neighbor_address: 192.0.2.5
#           address_family:
#             - afi: "ipv4"
#               safi: "unicast"
#               multipath: true
#               default_originate:
#                 set: true
#               capability_orf_prefix: both
#
# After state:
# -------------
# Nexus9000v# show running-config router bgp
# router bgp 65536
#  bgp router-id 192.0.1.1
#  address-family ipv4 unicast
#  address-family vpnv4 unicast
#  neighbor 192.0.2.2
#   remote-as 65537
#   address-family ipv4 unicast
#     multipath
#     weight 4
#     default-originate
#  neighbor 192.0.2.3
#   remote-as 65538
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  vrf vrf1
#   rd auto
#   neighbor 192.0.2.4
#    remote-as 65539
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate
#  vrf vrf2
#   rd auto
#   neighbor 192.0.2.5
#    remote-as 65540
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate
#
#
# Using overridden
# Before state:
# -------------
# RP/0/0/CPU0:iosxr-02#show running-config router bgp
# Sat Feb 20 03:49:43.618 UTC
# router bgp 65536
#  bgp router-id 192.0.1.1
#  address-family ipv4 unicast
#  address-family vpnv4 unicast
#  neighbor 192.0.2.2
#   remote-as 65537
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  neighbor 192.0.2.3
#   remote-as 65538
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  vrf vrf1
#   rd auto
#   neighbor 192.0.2.4
#    remote-as 65539
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate
#  vrf vrf2
#   rd auto
#   neighbor 192.0.2.5
#    remote-as 65540
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate

- name: override the provided configuration
  cisco.iosxr.iosxr_bgp_neighbor_address_family:
    state: overridden
    config:
      as_number: 65536
      neighbors:
        - neighbor_address: 192.0.2.2
          address_family:
            - afi: ipv4
              safi: unicast
              multipath: true
              default_originate:
                set: true
              weight: 5
# Task output
# -------------
#
# commands:
# - router bgp 65536
# - neighbor 192.0.2.3
# - no address-family ipv4 unicast
# - vrf vrf1
# - neighbor 192.0.2.4
# - no address-family ipv4 unicast
# - vrf vrf1
# - neighbor 192.0.2.5
# - no address-family ipv4 unicast
#
#
#
# after:
#   as_number: 65536
#   neighbors:
#     - neighbor_address: 192.0.2.2
#       address_family:
#         - afi: "ipv4"
#           safi: "unicast"
#           multipath: true
#           default_originate:
#             set: true
#           weight: 5
#
# After state:
# -------------
# RP/0/0/CPU0:iosxr-02#show running-config router bgp
# Sat Feb 20 03:49:43.618 UTC
# router bgp 65536
#  bgp router-id 192.0.1.1
#  address-family ipv4 unicast
#  address-family vpnv4 unicast
#  neighbor 192.0.2.2
#   remote-as 65537
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#
#
#
# Using rendered
# Before state:
# -------------
# RP/0/0/CPU0:iosxr-02#show running-config router bgp
# Sat Feb 20 03:49:43.618 UTC
# router bgp 65536
#  bgp router-id 192.0.2.1
#  address-family ipv4 unicast
#  address-family vpnv4 unicast
#  neighbor 192.0.2.2
#   remote-as 65537
#  neighbor 192.0.2.3
#   remote-as 65538
#  vrf vrf1
#   rd auto
#   neighbor 192.0.2.4
#    remote-as 65539
#  vrf vrf2
#   rd auto
#   neighbor 192.0.2.5
#    remote-as 65540

- name: >-
    Render platform specific configuration lines with state rendered (without
    connecting to the device)
  cisco.iosxr.iosxr_bgp_neighbor_address_family:
    state: rendered
    config:
      as_number: 65536
      neighbors:
        - neighbor_address: 192.0.2.2
          address_family:
            - afi: ipv4
              safi: unicast
              multipath: true
              default_originate:
                set: true
              weight: 5
        - neighbor_address: 192.0.2.3
          address_family:
            - afi: ipv4
              safi: unicast
              multipath: true
              default_originate:
                set: true
              weight: 4
      vrfs:
        - vrf: vrf1
          neighbors:
            - neighbor_address: 192.0.2.4
              address_family:
                - afi: ipv4
                  safi: unicast
                  multipath: true
                  default_originate:
                    set: true
                  capability_orf_prefix: both
        - vrf: vrf2
          neighbors:
            - neighbor_address: 192.0.2.5
              address_family:
                - afi: ipv4
                  safi: unicast
                  multipath: true
                  default_originate:
                    set: true
                  capability_orf_prefix: both
# Task output
# -------------
# commands:
# - router bgp 65536
# - neighbor 192.0.2.2
# - address-family ipv4 unicast
# - default-originate
# - multipath
# - weight 5
# - neighbor 192.0.2.3
# - address-family ipv4 unicast
# - default-originate
# - multipath
# - weight 4
# - vrf vrf1
# - neighbor 192.0.2.4
# - address-family ipv4 unicast
# - capability orf prefix both
# - default-originate
# - multipath
# - vrf vrf2
# - neighbor 192.0.2.5
# - address-family ipv4 unicast
# - capability orf prefix both
# - default-originate
# - multipath
#
# Using parsed
#
# parsed.cfg
# ------------
# router bgp 65536
#  bgp router-id 192.0.1.1
#  address-family ipv4 unicast
#  address-family vpnv4 unicast
#  neighbor 192.0.2.2
#   remote-as 65537
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  neighbor 1.1.1.2
#   remote-as 65538
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  vrf vrf1
#   rd auto
#   neighbor 192.0.2.4
#    remote-as 65539
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate
#  vrf vrf2
#   rd auto
#   neighbor 192.0.2.5
#    remote-as 65540
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate

- name: Parse externally provided BGP neighbor AF config
  cisco.iosxr.iosxr_bgp_neighbor_address_family:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed
# Task output (redacted)
# -----------------------
# parsed:
#   as_number: 65536
#   neighbors:
#     - neighbor_address: 192.0.2.2
#       address_family:
#         - afi: "ipv4"
#           safi: "unicast"
#           multipath: true
#           default_originate:
#             set: true
#           weight: 5
#     - neighbor_address: 192.0.2.3
#       address_family:
#         - afi: "ipv4"
#           safi: "unicast"
#           multipath: true
#           default_originate:
#             set: true
#           weight: 4
#   vrfs:
#     - vrf: vrf1
#       neighbors:
#         - neighbor_address: 192.0.2.4
#           address_family:
#             - afi: "ipv4"
#               safi: "unicast"
#               multipath: true
#               default_originate:
#                 set: true
#               capability_orf_prefix: both
#     - vrf: vrf2
#       neighbors:
#         - neighbor_address: 192.0.2.5
#           address_family:
#             - afi: "ipv4"
#               safi: "unicast"
#               multipath: true
#               default_originate:
#                 set: true
#               capability_orf_prefix: both
#
#
# Using Gathered
# -----------------
# Before state state:
# -------------
# RP/0/0/CPU0:iosxr-02#show running-config router bgp
# Sat Feb 20 03:49:43.618 UTC
# router bgp 65536
#  bgp router-id 192.0.1.1
#  address-family ipv4 unicast
#  address-family vpnv4 unicast
#  neighbor 192.0.2.2
#   remote-as 65537
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  neighbor 1.1.1.2
#   remote-as 65538
#   address-family ipv4 unicast
#     multipath
#     weight 5
#     default-originate
#  vrf vrf1
#   rd auto
#   neighbor 192.0.2.4
#    remote-as 65539
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate
#  vrf vrf2
#   rd auto
#   neighbor 192.0.2.5
#    remote-as 65540
#    address-family ipv4 unicast
#     multipath
#     capability orf prefix both
#     default-originate
#
#
#
- name: Gathered the provided configuration with the existing running configuration
  cisco.iosxr.iosxr_bgp_neighbor_address_family:
    config:
    state: gathered


# Task output
# -----------------------
# gathered:
#   as_number: 65536
#   neighbors:
#     - neighbor_address: 192.0.2.2
#       address_family:
#         - afi: "ipv4"
#           safi: "unicast"
#           multipath: true
#           default_originate:
#             set: true
#           weight: 5
#     - neighbor_address: 192.0.2.3
#       address_family:
#         - afi: "ipv4"
#           safi: "unicast"
#           multipath: true
#           default_originate:
#             set: true
#           weight: 4
#   vrfs:
#     - vrf: vrf1
#       neighbors:
#         - neighbor_address: 192.0.2.4
#           address_family:
#             - afi: "ipv4"
#               safi: "unicast"
#               multipath: true
#               default_originate:
#                 set: true
#               capability_orf_prefix: both
#     - vrf: vrf2
#       neighbors:
#         - neighbor_address: 192.0.2.5
#           address_family:
#             - afi: "ipv4"
#               safi: "unicast"
#               multipath: true
#               default_originate:
#                 set: true
#               capability_orf_prefix: both
#
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.bgp_neighbor_address_family.bgp_neighbor_address_family import (
    Bgp_neighbor_address_familyArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.bgp_neighbor_address_family.bgp_neighbor_address_family import (
    Bgp_neighbor_address_family,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Bgp_neighbor_address_familyArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Bgp_neighbor_address_family(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()
